@@ -192,7 +192,7 @@ static int i810_map_buffer(drm_buf_t *buf, struct file *filp)
 	if(buf_priv->currently_mapped == I810_BUF_MAPPED) return -EINVAL;
 
 	if(VM_DONTCOPY != 0) {
-		down(&current->mm->mmap_sem);
+		down_write(&current->mm->mmap_sem);
 		old_fops = filp->f_op;
 		filp->f_op = &i810_buffer_fops;
 		dev_priv->mmap_buffer = buf;
@@ -208,7 +208,7 @@ static int i810_map_buffer(drm_buf_t *buf, struct file *filp)
 			retcode = (signed int)buf_priv->virtual;
 			buf_priv->virtual = 0;
 		}
-   		up(&current->mm->mmap_sem);
+   		up_write(&current->mm->mmap_sem);
 	} else {
 		buf_priv->virtual = buf_priv->kernel_virtual;
    		buf_priv->currently_mapped = I810_BUF_MAPPED;
@@ -224,7 +224,7 @@ static int i810_unmap_buffer(drm_buf_t *buf)
 	if(VM_DONTCOPY != 0) {
 		if(buf_priv->currently_mapped != I810_BUF_MAPPED) 
 			return -EINVAL;
-		down(&current->mm->mmap_sem);
+		down_write(&current->mm->mmap_sem);
 #if LINUX_VERSION_CODE < 0x020399
         	retcode = do_munmap((unsigned long)buf_priv->virtual, 
 				    (size_t) buf->total);
@@ -233,7 +233,7 @@ static int i810_unmap_buffer(drm_buf_t *buf)
 				    (unsigned long)buf_priv->virtual, 
 				    (size_t) buf->total);
 #endif
-   		up(&current->mm->mmap_sem);
+   		up_write(&current->mm->mmap_sem);
 	}
    	buf_priv->currently_mapped = I810_BUF_UNMAPPED;
    	buf_priv->virtual = 0;
@@ -1286,6 +1286,8 @@ int i810_dma_vertex(struct inode *inode, struct file *filp,
 	DRM_DEBUG("i810 dma vertex, idx %d used %d discard %d\n",
 		  vertex.idx, vertex.used, vertex.discard);
 
+	if(vertex.idx < 0 || vertex.idx > dma->buf_count) return -EINVAL;
+
 	i810_dma_dispatch_vertex( dev, 
 				  dma->buflist[ vertex.idx ], 
 				  vertex.discard, vertex.used );
@@ -1409,7 +1411,7 @@ int i810_copybuf(struct inode *inode, struct file *filp, unsigned int cmd,
    	if (copy_from_user(&d, (drm_i810_copy_t *)arg, sizeof(d)))
 		return -EFAULT;
 
-	if(d.idx > dma->buf_count) return -EINVAL;
+	if(d.idx < 0 || d.idx > dma->buf_count) return -EINVAL;
 	buf = dma->buflist[ d.idx ];
    	buf_priv = buf->dev_private;
 	if (buf_priv->currently_mapped != I810_BUF_MAPPED) return -EPERM;
