@@ -27,10 +27,11 @@
  *
  *      Copyright (C) 1999,2000  Moxa Technologies Co., LTD.
  *
- *      for             : LINUX 2.0.X, 2.2.X
- *      date            : 1999/07/22
- *      version         : 1.1 
+ *      for             : LINUX 2.0.X, 2.2.X, 2.4.X
+ *      date            : 2001/05/01
+ *      version         : 1.2 
  *      
+ *    Fixes for C104H/PCI by Tim Hockin <thockin@sun.com>
  */
 
 #include <linux/config.h>
@@ -61,7 +62,7 @@
 #include <asm/bitops.h>
 #include <asm/uaccess.h>
 
-#define		MXSER_VERSION			"1.1kern"
+#define		MXSER_VERSION			"1.2"
 
 #define		MXSERMAJOR	 	174
 #define		MXSERCUMAJOR		175
@@ -120,7 +121,7 @@
 #define CI104J_ASIC_ID  5
 
 enum {
-	MXSER_BOARD_C168_ISA = 1,
+	MXSER_BOARD_C168_ISA = 0,
 	MXSER_BOARD_C104_ISA,
 	MXSER_BOARD_CI104J,
 	MXSER_BOARD_C168_PCI,
@@ -170,15 +171,10 @@ static mxser_pciinfo mxser_pcibrds[] =
 	{PCI_VENDOR_ID_MOXA, PCI_DEVICE_ID_C104, MXSER_BOARD_C104_PCI},
 };
 
-typedef struct _moxa_pci_info {
-	unsigned short busNum;
-	unsigned short devNum;
-} moxa_pci_info;
-
-static int ioaddr[MXSER_BOARDS] = {0, 0, 0, 0};
+static int ioaddr[MXSER_BOARDS];
 static int ttymajor = MXSERMAJOR;
 static int calloutmajor = MXSERCUMAJOR;
-static int verbose = 0;
+static int verbose;
 
 /* Variables for insmod */
 
@@ -198,7 +194,7 @@ struct mxser_hwconf {
 	int uart_type;
 	int ioaddr[MXSER_PORTS_PER_BOARD];
 	int baud_base[MXSER_PORTS_PER_BOARD];
-	moxa_pci_info pciInfo;
+	struct pci_dev *pdev;
 };
 
 struct mxser_struct {
@@ -279,7 +275,7 @@ static int mxvar_diagflag;
  * buffer across all the serial ports, since it significantly saves
  * memory if large numbers of serial ports are open.
  */
-static unsigned char *mxvar_tmp_buf = 0;
+static unsigned char *mxvar_tmp_buf;
 static struct semaphore mxvar_tmp_buf_sem;
 
 /*
@@ -304,7 +300,7 @@ void cleanup_module(void);
 static void mxser_getcfg(int board, struct mxser_hwconf *hwconf);
 int mxser_init(void);
 static int mxser_get_ISA_conf(int, struct mxser_hwconf *);
-static int mxser_get_PCI_conf(int, int, int, struct mxser_hwconf *);
+static int mxser_get_PCI_conf(struct pci_dev *, int, struct mxser_hwconf *);
 static void mxser_do_softint(void *);
 static int mxser_open(struct tty_struct *, struct file *);
 static void mxser_close(struct tty_struct *, struct file *);
@@ -439,7 +435,7 @@ int mxser_initbrd(int board, struct mxser_hwconf *hwconf)
 			     "mxser", info);
 	if (retval) {
 		restore_flags(flags);
-		printk("Board %d: %s", board, mxser_brdname[hwconf->board_type - 1]);
+		printk("Board %d: %s", board, mxser_brdname[hwconf->board_type]);
 		printk("  Request irq fail,IRQ (%d) may be conflit with another device.\n", info->irq);
 		return (retval);
 	}
@@ -454,33 +450,21 @@ static void mxser_getcfg(int board, struct mxser_hwconf *hwconf)
 	mxsercfg[board] = *hwconf;
 }
 
-static int mxser_get_PCI_conf(int busnum, int devnum, int board_type, struct mxser_hwconf *hwconf)
+static int mxser_get_PCI_conf(struct pci_dev *pdev, int board_type, struct mxser_hwconf *hwconf)
 {
 	int i;
-	unsigned int val, ioaddress;
+	unsigned int ioaddress;
 
 	hwconf->board_type = board_type;
-	hwconf->ports = mxser_numports[board_type - 1];
-	pcibios_read_config_dword(busnum, devnum, PCI_BASE_ADDRESS_2, &val);
-	if (val == 0xffffffff)
-		return (MXSER_ERR_IOADDR);
-	else
-		ioaddress = val & 0xffffffc;
+	hwconf->ports = mxser_numports[board_type];
+	ioaddress = pci_resource_start (pdev, 2);
 	for (i = 0; i < hwconf->ports; i++)
 		hwconf->ioaddr[i] = ioaddress + 8 * i;
 
-	pcibios_read_config_dword(busnum, devnum, PCI_BASE_ADDRESS_3, &val);
-	if (val == 0xffffffff)
-		return (MXSER_ERR_VECTOR);
-	else
-		ioaddress = val & 0xffffffc;
+	ioaddress = pci_resource_start (pdev, 3);
 	hwconf->vector = ioaddress;
 
-	pcibios_read_config_dword(busnum, devnum, PCI_INTERRUPT_LINE, &val);
-	if (val == 0xffffffff)
-		return (MXSER_ERR_IRQ);
-	else
-		hwconf->irq = val & 0xff;
+	hwconf->irq = pdev->irq;
 
 	hwconf->uart_type = PORT_16550A;
 	hwconf->vector_mask = 0;
@@ -496,7 +480,6 @@ int mxser_init(void)
 	int i, m, retval, b;
 	int n, index;
 	int ret1, ret2;
-	unsigned char busnum, devnum;
 	struct mxser_hwconf hwconf;
 
 	printk("MOXA Smartio family driver version %s\n", MXSER_VERSION);
@@ -562,7 +545,7 @@ int mxser_init(void)
 
 		if (retval != 0)
 			printk("Found MOXA %s board (CAP=0x%x)\n",
-			       mxser_brdname[hwconf.board_type - 1],
+			       mxser_brdname[hwconf.board_type],
 			       ioaddr[b]);
 
 		if (retval <= 0) {
@@ -577,8 +560,7 @@ int mxser_init(void)
 
 			continue;
 		}
-		hwconf.pciInfo.busNum = 0;
-		hwconf.pciInfo.devNum = 0;
+		hwconf.pdev = NULL;
 
 		if (mxser_initbrd(m, &hwconf) < 0)
 			continue;
@@ -598,7 +580,7 @@ int mxser_init(void)
 
 		if (retval != 0)
 			printk("Found MOXA %s board (CAP=0x%x)\n",
-			       mxser_brdname[hwconf.board_type - 1],
+			       mxser_brdname[hwconf.board_type],
 			       ioaddr[b]);
 
 		if (retval <= 0) {
@@ -613,8 +595,7 @@ int mxser_init(void)
 
 			continue;
 		}
-		hwconf.pciInfo.busNum = 0;
-		hwconf.pciInfo.devNum = 0;
+		hwconf.pdev = NULL;
 
 		if (mxser_initbrd(m, &hwconf) < 0)
 			continue;
@@ -627,29 +608,24 @@ int mxser_init(void)
 	/* start finding PCI board here */
 
 #ifdef CONFIG_PCI
-	if (pci_present()) 
 	{
+		struct pci_dev *pdev = NULL;
+
 		n = sizeof(mxser_pcibrds) / sizeof(mxser_pciinfo);
 		index = 0;
-		b = 0;
-		while (b < n) {
-			if (pcibios_find_device(mxser_pcibrds[b].vendor_id,
-					      mxser_pcibrds[b].device_id,
-						index,
-						&busnum,
-						&devnum) != 0) {
-				b++;
-				index = 0;
+		for (b = 0; b < n; b++) {
+			pdev = pci_find_device(mxser_pcibrds[b].vendor_id,
+					       mxser_pcibrds[b].device_id, pdev);
+			if (!pdev || pci_enable_device(pdev))
 				continue;
-			}
-			hwconf.pciInfo.busNum = busnum;
-			hwconf.pciInfo.devNum = devnum;
-			printk("Found MOXA %s board(BusNo=%d,DevNo=%d)\n", mxser_brdname[mxser_pcibrds[b].board_type - 1], busnum, devnum >> 3);
-			index++;
+			hwconf.pdev = pdev;
+			printk("Found MOXA %s board(BusNo=%d,DevNo=%d)\n",
+				mxser_brdname[mxser_pcibrds[b].board_type],
+				pdev->bus->number, PCI_SLOT(pdev->devfn));
 			if (m >= MXSER_BOARDS) {
-				printk("Too many Smartio family boards find (maximum %d),board not configured\n", MXSER_BOARDS);
+				printk("Too many Smartio family boards found (maximum %d),board not configured\n", MXSER_BOARDS);
 			} else {
-				retval = mxser_get_PCI_conf(busnum, devnum,
+				retval = mxser_get_PCI_conf(pdev,
 				   mxser_pcibrds[b].board_type, &hwconf);
 				if (retval < 0) {
 					if (retval == MXSER_ERR_IRQ)
@@ -711,17 +687,18 @@ static void mxser_do_softint(void *private_)
 	struct tty_struct *tty;
 
 	tty = info->tty;
-	if (!tty)
-		return;
-	if (test_and_clear_bit(MXSER_EVENT_TXLOW, &info->event)) {
-		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
-		    tty->ldisc.write_wakeup)
-			(tty->ldisc.write_wakeup) (tty);
-		wake_up_interruptible(&tty->write_wait);
-	}
-	if (test_and_clear_bit(MXSER_EVENT_HANGUP, &info->event)) {
-		tty_hangup(tty);
+	if (tty) {
+		if (test_and_clear_bit(MXSER_EVENT_TXLOW, &info->event)) {
+			if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
+			    tty->ldisc.write_wakeup)
+				(tty->ldisc.write_wakeup) (tty);
+			wake_up_interruptible(&tty->write_wait);
+		}
+		if (test_and_clear_bit(MXSER_EVENT_HANGUP, &info->event)) {
+			tty_hangup(tty);	/* FIXME: module removal race here - AKPM */
+		}
 	}
+	MOD_DEC_USE_COUNT;
 }
 
 /*
@@ -910,29 +887,56 @@ static int mxser_write(struct tty_struct *tty, int from_user,
 	if (from_user)
 		down(&mxvar_tmp_buf_sem);
 	save_flags(flags);
-	while (1) {
-		cli();
-		c = MIN(count, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
-				   SERIAL_XMIT_SIZE - info->xmit_head));
-		if (c <= 0)
-			break;
+	if (from_user) {
+		down(&mxvar_tmp_buf_sem);
+		while (1) {
+			c = MIN(count, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
+					   SERIAL_XMIT_SIZE - info->xmit_head));
+			if (c <= 0)
+				break;
 
-		if (from_user) {
-			copy_from_user(mxvar_tmp_buf, buf, c);
+			c -= copy_from_user(mxvar_tmp_buf, buf, c);
+			if (!c) {
+				if (!total)
+					total = -EFAULT;
+				break;
+			}
+
+			cli();
 			c = MIN(c, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
-				    SERIAL_XMIT_SIZE - info->xmit_head));
+				       SERIAL_XMIT_SIZE - info->xmit_head));
 			memcpy(info->xmit_buf + info->xmit_head, mxvar_tmp_buf, c);
-		} else
-			memcpy(info->xmit_buf + info->xmit_head, buf, c);
-		info->xmit_head = (info->xmit_head + c) & (SERIAL_XMIT_SIZE - 1);
-		info->xmit_cnt += c;
-		restore_flags(flags);
-		buf += c;
-		count -= c;
-		total += c;
-	}
-	if (from_user)
+			info->xmit_head = (info->xmit_head + c) & (SERIAL_XMIT_SIZE - 1);
+			info->xmit_cnt += c;
+			restore_flags(flags);
+
+			buf += c;
+			count -= c;
+			total += c;
+		}
 		up(&mxvar_tmp_buf_sem);
+	} else {
+		while (1) {
+			cli();
+			c = MIN(count, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
+					   SERIAL_XMIT_SIZE - info->xmit_head));
+			if (c <= 0) {
+				restore_flags(flags);
+				break;
+			}
+
+			memcpy(info->xmit_buf + info->xmit_head, buf, c);
+			info->xmit_head = (info->xmit_head + c) & (SERIAL_XMIT_SIZE - 1);
+			info->xmit_cnt += c;
+			restore_flags(flags);
+
+			buf += c;
+			count -= c;
+			total += c;
+		}
+	}
+
+	cli();
 	if (info->xmit_cnt && !tty->stopped && !tty->hw_stopped &&
 	    !(info->IER & UART_IER_THRI)) {
 		info->IER |= UART_IER_THRI;
@@ -1370,7 +1374,7 @@ static void mxser_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 	if (port == 0)
 		return;
-	max = mxser_numports[mxsercfg[i].board_type - 1];
+	max = mxser_numports[mxsercfg[i].board_type];
 
 	while (1) {
 		irqbits = inb(port->vector) & port->vectormask;
@@ -1477,7 +1481,9 @@ static inline void mxser_transmit_chars(struct mxser_struct *info)
 
 	if (info->xmit_cnt < WAKEUP_CHARS) {
 		set_bit(MXSER_EVENT_TXLOW, &info->event);
-		queue_task(&info->tqueue, &tq_scheduler);
+		MOD_INC_USE_COUNT;
+		if (schedule_task(&info->tqueue) == 0)
+		    MOD_DEC_USE_COUNT;
 	}
 	if (info->xmit_cnt <= 0) {
 		info->IER &= ~UART_IER_THRI;
@@ -1506,8 +1512,9 @@ static inline void mxser_check_modem_status(struct mxser_struct *info,
 		else if (!((info->flags & ASYNC_CALLOUT_ACTIVE) &&
 			   (info->flags & ASYNC_CALLOUT_NOHUP)))
 			set_bit(MXSER_EVENT_HANGUP, &info->event);
-		queue_task(&info->tqueue, &tq_scheduler);
-
+		MOD_INC_USE_COUNT;
+		if (schedule_task(&info->tqueue) == 0)
+		    MOD_DEC_USE_COUNT;
 	}
 	if (info->flags & ASYNC_CTS_FLOW) {
 		if (info->tty->hw_stopped) {
@@ -1517,7 +1524,9 @@ static inline void mxser_check_modem_status(struct mxser_struct *info,
 				outb(info->IER, info->base + UART_IER);
 
 				set_bit(MXSER_EVENT_TXLOW, &info->event);
-				queue_task(&info->tqueue, &tq_scheduler);
+				MOD_INC_USE_COUNT;
+				if (schedule_task(&info->tqueue) == 0)
+					MOD_DEC_USE_COUNT;
 			}
 		} else {
 			if (!(status & UART_MSR_CTS)) {
@@ -1689,7 +1698,7 @@ static int mxser_startup(struct mxser_struct *info)
 	 */
 	if (inb(info->base + UART_LSR) == 0xff) {
 		restore_flags(flags);
-		if (suser()) {
+		if (capable(CAP_SYS_ADMIN)) {
 			if (info->tty)
 				set_bit(TTY_IO_ERROR, &info->tty->flags);
 			return (0);
@@ -2206,8 +2215,7 @@ static int mxser_get_lsr_info(struct mxser_struct *info, unsigned int *value)
 	status = inb(info->base + UART_LSR);
 	restore_flags(flags);
 	result = ((status & UART_LSR_TEMT) ? TIOCSER_TEMT : 0);
-	put_user(result, value);
-	return (0);
+	return put_user(result, value);
 }
 
 /*
@@ -2247,8 +2255,7 @@ static int mxser_get_modem_info(struct mxser_struct *info,
 	    ((status & UART_MSR_RI) ? TIOCM_RNG : 0) |
 	    ((status & UART_MSR_DSR) ? TIOCM_DSR : 0) |
 	    ((status & UART_MSR_CTS) ? TIOCM_CTS : 0);
-	put_user(result, value);
-	return (0);
+	return put_user(result, value);
 }
 
 static int mxser_set_modem_info(struct mxser_struct *info, unsigned int cmd,
