@@ -99,7 +99,7 @@
 #include <linux/ptrace.h>
 #include <linux/errno.h>
 #include <linux/in.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/ioport.h>
 #include <linux/spinlock.h>
 #include <asm/bitops.h>
@@ -119,26 +119,26 @@
  *
  *********************************************************/
 
-static const char *filename = __FILE__;
+static const char filename[] = __FILE__;
 
-static const char *timeout_msg = "*** timeout at %s:%s (line %d) ***\n";
+static const char timeout_msg[] = "*** timeout at %s:%s (line %d) ***\n";
 #define TIMEOUT_MSG(lineno) \
 	printk(timeout_msg, filename,__FUNCTION__,(lineno))
 
-static const char *invalid_pcb_msg =
+static const char invalid_pcb_msg[] =
 "*** invalid pcb length %d at %s:%s (line %d) ***\n";
 #define INVALID_PCB_MSG(len) \
 	printk(invalid_pcb_msg, (len),filename,__FUNCTION__,__LINE__)
 
-static const char *search_msg = "%s: Looking for 3c505 adapter at address %#x...";
+static char search_msg[] __initdata = "%s: Looking for 3c505 adapter at address %#x...";
 
-static const char *stilllooking_msg = "still looking...";
+static char stilllooking_msg[] __initdata = "still looking...";
 
-static const char *found_msg = "found.\n";
+static char found_msg[] __initdata = "found.\n";
 
-static const char *notfound_msg = "not found (reason = %d)\n";
+static char notfound_msg[] __initdata = "not found (reason = %d)\n";
 
-static const char *couldnot_msg = "%s: 3c505 not found\n";
+static char couldnot_msg[] __initdata = "%s: 3c505 not found\n";
 
 /*********************************************************
  *
@@ -149,7 +149,7 @@ static const char *couldnot_msg = "%s: 3c505 not found\n";
 #ifdef ELP_DEBUG
 static const int elp_debug = ELP_DEBUG;
 #else
-static const int elp_debug = 0;
+static const int elp_debug;
 #endif
 
 /*
@@ -180,7 +180,7 @@ static const int elp_debug = 0;
  * Last element MUST BE 0!
  *****************************************************************/
 
-static const int addr_list[] __initdata = {0x300, 0x280, 0x310, 0};
+static int addr_list[] __initdata = {0x300, 0x280, 0x310, 0};
 
 /* Dma Memory related stuff */
 
@@ -601,15 +601,16 @@ static void receive_packet(struct net_device *dev, int len)
 		printk("%s: memory squeeze, dropping packet\n", dev->name);
 		target = adapter->dma_buffer;
 		adapter->current_dma.target = NULL;
+		return;
+	}
+
+	skb_reserve(skb, 2);
+	target = skb_put(skb, rlen);
+	if (virt_to_bus(target + rlen) >= MAX_DMA_ADDRESS) {
+		adapter->current_dma.target = target;
+		target = adapter->dma_buffer;
 	} else {
-		skb_reserve(skb, 2);
-		target = skb_put(skb, rlen);
-		if (virt_to_bus(target + rlen) >= MAX_DMA_ADDRESS) {
-			adapter->current_dma.target = target;
-			target = adapter->dma_buffer;
-		} else {
-			adapter->current_dma.target = NULL;
-		}
+		adapter->current_dma.target = NULL;
 	}
 
 	/* if this happens, we die */
@@ -689,6 +690,7 @@ static void elp_interrupt(int irq, void *dev_id, struct pt_regs *reg_ptr)
 					skb->protocol = eth_type_trans(skb,dev);
 					adapter->stats.rx_bytes += skb->len;
 					netif_rx(skb);
+					dev->last_rx = jiffies;
 				}
 			}
 			adapter->dmaing = 0;
@@ -854,6 +856,7 @@ static void elp_interrupt(int irq, void *dev_id, struct pt_regs *reg_ptr)
 static int elp_open(struct net_device *dev)
 {
 	elp_device *adapter;
+	int retval;
 
 	adapter = dev->priv;
 
@@ -893,16 +896,21 @@ static int elp_open(struct net_device *dev)
 	/*
 	 * install our interrupt service routine
 	 */
-	if (request_irq(dev->irq, &elp_interrupt, 0, "3c505", dev)) {
-		return -EAGAIN;
+	if ((retval = request_irq(dev->irq, &elp_interrupt, 0, dev->name, dev))) {
+		printk(KERN_ERR "%s: could not allocate IRQ%d\n", dev->name, dev->irq);
+		return retval;
 	}
-	if (request_dma(dev->dma, "3c505")) {
-		printk("%s: could not allocate DMA channel\n", dev->name);
-		return -EAGAIN;
+	if ((retval = request_dma(dev->dma, dev->name))) {
+		free_irq(dev->irq, dev);
+		printk(KERN_ERR "%s: could not allocate DMA%d channel\n", dev->name, dev->dma);
+		return retval;
 	}
 	adapter->dma_buffer = (void *) dma_mem_alloc(DMA_BUFFER_SIZE);
 	if (!adapter->dma_buffer) {
-		printk("Could not allocate DMA buffer\n");
+		printk(KERN_ERR "%s: could not allocate DMA buffer\n", dev->name);
+		free_dma(dev->dma);
+		free_irq(dev->irq, dev);
+		return -ENOMEM;
 	}
 	adapter->dmaing = 0;
 
@@ -915,7 +923,7 @@ static int elp_open(struct net_device *dev)
 	 * configure adapter memory: we need 10 multicast addresses, default==0
 	 */
 	if (elp_debug >= 3)
-		printk("%s: sending 3c505 memory configuration command\n", dev->name);
+		printk(KERN_DEBUG "%s: sending 3c505 memory configuration command\n", dev->name);
 	adapter->tx_pcb.command = CMD_CONFIGURE_ADAPTER_MEMORY;
 	adapter->tx_pcb.data.memconf.cmd_q = 10;
 	adapter->tx_pcb.data.memconf.rcv_q = 20;
@@ -967,10 +975,8 @@ static int elp_open(struct net_device *dev)
 	 * device is now officially open!
 	 */
 
-	netif_wake_queue(dev);
-	MOD_INC_USE_COUNT;
-
-	return 0;		/* Always succeed */
+	netif_start_queue(dev);
+	return 0;
 }
 
 
@@ -1049,7 +1055,6 @@ static int send_packet(struct net_device *dev, struct sk_buff *skb)
  
 static void elp_timeout(struct net_device *dev)
 {
-	unsigned long flags;
 	elp_device *adapter = dev->priv;
 	int stat;
 
@@ -1179,8 +1184,6 @@ static int elp_close(struct net_device *dev)
 	free_dma(dev->dma);
 	free_pages((unsigned long) adapter->dma_buffer, get_order(DMA_BUFFER_SIZE));
 
-	MOD_DEC_USE_COUNT;
-
 	return 0;
 }
 
@@ -1303,8 +1306,8 @@ static int __init elp_sense(struct net_device *dev)
 	long flags;
 	byte orig_HSR;
 
-	if (check_region(addr, 0xf))
-		return -1;
+	if (!request_region(addr, ELP_IO_EXTENT, "3c505"))
+		return -ENODEV;
 
 	orig_HSR = inb_status(addr);
 
@@ -1314,7 +1317,7 @@ static int __init elp_sense(struct net_device *dev)
 	if (orig_HSR == 0xff) {
 		if (elp_debug > 0)
 			printk(notfound_msg, 1);
-		return -1;
+		goto out;
 	}
 	/* Enable interrupts - we need timers! */
 	save_flags(flags);
@@ -1333,7 +1336,7 @@ static int __init elp_sense(struct net_device *dev)
 		if (inb_status(addr) & DIR) {
 			if (elp_debug > 0)
 				printk(notfound_msg, 2);
-			return -1;
+			goto out;
 		}
 	} else {
 		/* If HCR.DIR is down, we pull it up. HSR.DIR should follow. */
@@ -1344,7 +1347,7 @@ static int __init elp_sense(struct net_device *dev)
 		if (!(inb_status(addr) & DIR)) {
 			if (elp_debug > 0)
 				printk(notfound_msg, 3);
-			return -1;
+			goto out;
 		}
 	}
 	/*
@@ -1354,6 +1357,9 @@ static int __init elp_sense(struct net_device *dev)
 		printk(found_msg);
 
 	return 0;
+out:
+	release_region(addr, ELP_IO_EXTENT);
+	return -ENODEV;
 }
 
 /*************************************************************
@@ -1410,6 +1416,9 @@ int __init elplus_probe(struct net_device *dev)
 {
 	elp_device *adapter;
 	int i, tries, tries1, timeout, okay;
+	unsigned long cookie = 0;
+
+	SET_MODULE_OWNER(dev);
 
 	/*
 	 *  setup adapter structure
@@ -1477,21 +1486,21 @@ int __init elplus_probe(struct net_device *dev)
 			 */
 			adapter->tx_pcb.command = CMD_STATION_ADDRESS;
 			adapter->tx_pcb.length = 0;
-			autoirq_setup(0);
+			cookie = probe_irq_on();
 			if (!send_pcb(dev, &adapter->tx_pcb)) {
 				printk("%s: could not send first PCB\n", dev->name);
-				autoirq_report(0);
+				probe_irq_off(cookie);
 				continue;
 			}
 			if (!receive_pcb(dev, &adapter->rx_pcb)) {
 				printk("%s: could not read first PCB\n", dev->name);
-				autoirq_report(0);
+				probe_irq_off(cookie);
 				continue;
 			}
 			if ((adapter->rx_pcb.command != CMD_ADDRESS_RESPONSE) ||
 			    (adapter->rx_pcb.length != 6)) {
 				printk("%s: first PCB wrong (%d, %d)\n", dev->name, adapter->rx_pcb.command, adapter->rx_pcb.length);
-				autoirq_report(0);
+				probe_irq_off(cookie);
 				continue;
 			}
 			goto okay;
@@ -1504,17 +1513,18 @@ int __init elplus_probe(struct net_device *dev)
 		outb_control(adapter->hcr_val & ~(FLSH | ATTN), dev);
 	}
 	printk("%s: failed to initialise 3c505\n", dev->name);
+	release_region(dev->base_addr, ELP_IO_EXTENT);
 	return -ENODEV;
 
       okay:
 	if (dev->irq) {		/* Is there a preset IRQ? */
-		int rpt = autoirq_report(0);
+		int rpt = probe_irq_off(cookie);
 		if (dev->irq != rpt) {
 			printk("%s: warning, irq %d configured but %d detected\n", dev->name, dev->irq, rpt);
 		}
-		/* if dev->irq == autoirq_report(0), all is well */
+		/* if dev->irq == probe_irq_off(cookie), all is well */
 	} else		       /* No preset IRQ; just use what we can detect */
-		dev->irq = autoirq_report(0);
+		dev->irq = probe_irq_off(cookie);
 	switch (dev->irq) {    /* Legal, sane? */
 	case 0:
 		printk("%s: IRQ probe failed: check 3c505 jumpers.\n",
@@ -1524,7 +1534,7 @@ int __init elplus_probe(struct net_device *dev)
 	case 6:
 	case 8:
 	case 13:
-		printk("%s: Impossible IRQ %d reported by autoirq_report().\n",
+		printk("%s: Impossible IRQ %d reported by probe_irq_off().\n",
 		       dev->name, dev->irq);
 		return -ENODEV;
 	}
@@ -1594,10 +1604,6 @@ int __init elplus_probe(struct net_device *dev)
 	if (adapter->rx_pcb.data.configure) {
 		printk("%s: adapter configuration failed\n", dev->name);
 	}
-	/*
-	 * and reserve the address region
-	 */
-	request_region(dev->base_addr, ELP_IO_EXTENT, "3c505");
 
 	/*
 	 * initialise the device
@@ -1608,22 +1614,16 @@ int __init elplus_probe(struct net_device *dev)
 }
 
 #ifdef MODULE
-#define NAMELEN 9
-static char devicename[ELP_MAX_CARDS][NAMELEN] = {{0,}};
-static struct net_device dev_3c505[ELP_MAX_CARDS] =
-{
-	{ NULL,		/* device name is inserted by net_init.c */
-	0, 0, 0, 0,
-	0, 0,
-	0, 0, 0, NULL, elplus_probe},
-};
-
-static int io[ELP_MAX_CARDS] = { 0, };
-static int irq[ELP_MAX_CARDS] = { 0, };
-static int dma[ELP_MAX_CARDS] = { 0, };
+static struct net_device dev_3c505[ELP_MAX_CARDS];
+static int io[ELP_MAX_CARDS];
+static int irq[ELP_MAX_CARDS];
+static int dma[ELP_MAX_CARDS];
 MODULE_PARM(io, "1-" __MODULE_STRING(ELP_MAX_CARDS) "i");
 MODULE_PARM(irq, "1-" __MODULE_STRING(ELP_MAX_CARDS) "i");
 MODULE_PARM(dma, "1-" __MODULE_STRING(ELP_MAX_CARDS) "i");
+MODULE_PARM_DESC(io, "EtherLink Plus I/O base address(es)");
+MODULE_PARM_DESC(irq, "EtherLink Plus IRQ number(s) (assigned)");
+MODULE_PARM_DESC(dma, "EtherLink Plus DMA channel(s)");
 
 int init_module(void)
 {
@@ -1631,9 +1631,9 @@ int init_module(void)
 
 	for (this_dev = 0; this_dev < ELP_MAX_CARDS; this_dev++) {
 		struct net_device *dev = &dev_3c505[this_dev];
-		dev->name = devicename[this_dev];
 		dev->irq = irq[this_dev];
 		dev->base_addr = io[this_dev];
+		dev->init = elplus_probe;
 		if (dma[this_dev]) {
 			dev->dma = dma[this_dev];
 		} else {
