@@ -16,7 +16,7 @@
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/string.h>
 #include <linux/errno.h>
 
@@ -33,19 +33,19 @@
  *	Index to functions.
  */
 
-int	    ethertap_probe(struct device *dev);
-static int  ethertap_open(struct device *dev);
-static int  ethertap_start_xmit(struct sk_buff *skb, struct device *dev);
-static int  ethertap_close(struct device *dev);
-static struct net_device_stats *ethertap_get_stats(struct device *dev);
+int	    ethertap_probe(struct net_device *dev);
+static int  ethertap_open(struct net_device *dev);
+static int  ethertap_start_xmit(struct sk_buff *skb, struct net_device *dev);
+static int  ethertap_close(struct net_device *dev);
+static struct net_device_stats *ethertap_get_stats(struct net_device *dev);
 static void ethertap_rx(struct sock *sk, int len);
 #ifdef CONFIG_ETHERTAP_MC
-static void set_multicast_list(struct device *dev);
+static void set_multicast_list(struct net_device *dev);
 #endif
 
-static int ethertap_debug = 0;
+static int ethertap_debug;
 
-static struct device *tap_map[32];	/* Returns the tap device for a given netlink */
+static struct net_device *tap_map[32];	/* Returns the tap device for a given netlink */
 
 /*
  *	Board-specific info in dev->priv.
@@ -65,8 +65,10 @@ struct net_local
  *	hardware it would have to check what was present.
  */
  
-__initfunc(int ethertap_probe(struct device *dev))
+int __init ethertap_probe(struct net_device *dev)
 {
+	SET_MODULE_OWNER(dev);
+
 	memcpy(dev->dev_addr, "\xFE\xFD\x00\x00\x00\x00", 6);
 	if (dev->mem_start & 0xf)
 		ethertap_debug = dev->mem_start & 0x7;
@@ -109,23 +111,17 @@ __initfunc(int ethertap_probe(struct device *dev))
  *	Open/initialize the board.
  */
 
-static int ethertap_open(struct device *dev)
+static int ethertap_open(struct net_device *dev)
 {
 	struct net_local *lp = (struct net_local*)dev->priv;
 
 	if (ethertap_debug > 2)
 		printk("%s: Doing ethertap_open()...", dev->name);
 
-	MOD_INC_USE_COUNT;
-
 	lp->nl = netlink_kernel_create(dev->base_addr, ethertap_rx);
-	if (lp->nl == NULL) {
-		MOD_DEC_USE_COUNT;
+	if (lp->nl == NULL)
 		return -ENOBUFS;
-	}
-
-	dev->start = 1;
-	dev->tbusy = 0;
+	netif_start_queue(dev);
 	return 0;
 }
 
@@ -142,7 +138,7 @@ static unsigned ethertap_mc_hash(__u8 *dest)
 	return 1U << (idx&0x1F);
 }
 
-static void set_multicast_list(struct device *dev)
+static void set_multicast_list(struct net_device *dev)
 {
 	unsigned groups = ~0;
 	struct net_local *lp = (struct net_local *)dev->priv;
@@ -169,7 +165,7 @@ static void set_multicast_list(struct device *dev)
  *	it for 2.0 so that we dev_kfree_skb() the locked original.
  */
  
-static int ethertap_start_xmit(struct sk_buff *skb, struct device *dev)
+static int ethertap_start_xmit(struct sk_buff *skb, struct net_device *dev)
 {
 	struct net_local *lp = (struct net_local *)dev->priv;
 #ifdef CONFIG_ETHERTAP_MC
@@ -227,7 +223,7 @@ static int ethertap_start_xmit(struct sk_buff *skb, struct device *dev)
 	return 0;
 }
 
-static __inline__ int ethertap_rx_skb(struct sk_buff *skb, struct device *dev)
+static __inline__ int ethertap_rx_skb(struct sk_buff *skb, struct net_device *dev)
 {
 	struct net_local *lp = (struct net_local *)dev->priv;
 #ifdef CONFIG_ETHERTAP_MC
@@ -283,6 +279,7 @@ static __inline__ int ethertap_rx_skb(struct sk_buff *skb, struct device *dev)
 	lp->stats.rx_packets++;
 	lp->stats.rx_bytes+=len;
 	netif_rx(skb);
+	dev->last_rx = jiffies;
 	return len;
 }
 
@@ -295,7 +292,7 @@ static __inline__ int ethertap_rx_skb(struct sk_buff *skb, struct device *dev)
 
 static void ethertap_rx(struct sock *sk, int len)
 {
-	struct device *dev = tap_map[sk->protocol];
+	struct net_device *dev = tap_map[sk->protocol];
 	struct sk_buff *skb;
 
 	if (dev==NULL) {
@@ -311,7 +308,7 @@ static void ethertap_rx(struct sock *sk, int len)
 		ethertap_rx_skb(skb, dev);
 }
 
-static int ethertap_close(struct device *dev)
+static int ethertap_close(struct net_device *dev)
 {
 	struct net_local *lp = (struct net_local *)dev->priv;
 	struct sock *sk = lp->nl;
@@ -319,19 +316,17 @@ static int ethertap_close(struct device *dev)
 	if (ethertap_debug > 2)
 		printk("%s: Shutting down.\n", dev->name);
 
-	dev->tbusy = 1;
-	dev->start = 0;
+	netif_stop_queue(dev);
 
 	if (sk) {
 		lp->nl = NULL;
 		sock_release(sk->socket);
 	}
 
-	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
-static struct net_device_stats *ethertap_get_stats(struct device *dev)
+static struct net_device_stats *ethertap_get_stats(struct net_device *dev)
 {
 	struct net_local *lp = (struct net_local *)dev->priv;
 	return &lp->stats;
@@ -341,12 +336,11 @@ static struct net_device_stats *ethertap_get_stats(struct device *dev)
 
 static int unit;
 MODULE_PARM(unit,"i");
+MODULE_PARM_DESC(unit,"Ethertap device number");
 
-static char devicename[9] = { 0, };
-
-static struct device dev_ethertap =
+static struct net_device dev_ethertap =
 {
-	devicename,
+	" ",
 	0, 0, 0, 0,
 	1, 5,
 	0, 0, 0, NULL, ethertap_probe
@@ -355,10 +349,10 @@ static struct device dev_ethertap =
 int init_module(void)
 {
 	dev_ethertap.base_addr=unit+NETLINK_TAPBASE;
-	sprintf(devicename,"tap%d",unit);
-	if (dev_get(devicename))
+	sprintf(dev_ethertap.name,"tap%d",unit);
+	if (dev_get(dev_ethertap.name))
 	{
-		printk(KERN_INFO "%s already loaded.\n", devicename);
+		printk(KERN_INFO "%s already loaded.\n", dev_ethertap.name);
 		return -EBUSY;
 	}
 	if (register_netdev(&dev_ethertap) != 0)
