@@ -96,7 +96,7 @@
  *
  * wavelan.h :		Description of the hardware interface & structs
  *
- * i82593.h :		Description if the Ethernet controler
+ * i82593.h :		Description if the Ethernet controller
  */
 
 /* --------------------------- HISTORY --------------------------- */
@@ -225,7 +225,7 @@
  *	- wavelan_set_multicast_list : avoid reset
  *	- add wireless extensions (ioctl & get_wireless_stats)
  *	  get/set nwid/frequency on fly, info for /proc/net/wireless
- *	- Supress useless stuff from lp (net_local), but add link
+ *	- Suppress useless stuff from lp (net_local), but add link
  *	- More inlines
  *	- Lot of others minor details & cleanups
  *
@@ -315,7 +315,7 @@
  *		o Rename wavelan_release to wv_pcmcia_release & move up
  *		o move unregister_netdev to wavelan_detach()
  *		o wavelan_release() no longer call wavelan_detach()
- *		o Supress "release" timer
+ *		o Suppress "release" timer
  *		o Other cleanups & fixes
  *	- New MAC address in the probe
  *	- Reorg PSA_CRC code (endian neutral & cleaner)
@@ -358,12 +358,13 @@
 /***************************** INCLUDES *****************************/
 
 /* Linux headers that we need */
+#include <linux/config.h>
 #include <linux/module.h>
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/string.h>
 #include <linux/timer.h>
 #include <linux/interrupt.h>
@@ -381,9 +382,9 @@
 #include <linux/ioport.h>
 #include <linux/fcntl.h>
 
-#ifdef CONFIG_NET_RADIO
+#ifdef CONFIG_NET_PCMCIA_RADIO
 #include <linux/wireless.h>		/* Wireless extensions */
-#endif
+#endif	/* CONFIG_NET_PCMCIA_RADIO */
 
 /* Pcmcia headers that we need */
 #include <pcmcia/cs_types.h>
@@ -429,14 +430,14 @@
 #undef DEBUG_CONFIG_INFO	/* What's going on... */
 #define DEBUG_CONFIG_ERRORS	/* Errors on configuration */
 #undef DEBUG_TX_TRACE		/* Transmission calls */
-#undef DEBUG_TX_INFO		/* Header of the transmited packet */
+#undef DEBUG_TX_INFO		/* Header of the transmitted packet */
 #undef DEBUG_TX_FAIL		/* Normal failure conditions */
 #define DEBUG_TX_ERROR		/* Unexpected conditions */
 #undef DEBUG_RX_TRACE		/* Transmission calls */
-#undef DEBUG_RX_INFO		/* Header of the transmited packet */
+#undef DEBUG_RX_INFO		/* Header of the transmitted packet */
 #undef DEBUG_RX_FAIL		/* Normal failure conditions */
 #define DEBUG_RX_ERROR		/* Unexpected conditions */
-#undef DEBUG_PACKET_DUMP	32	/* Dump packet on the screen */
+#undef DEBUG_PACKET_DUMP	/* Dump packet on the screen */
 #undef DEBUG_IOCTL_TRACE	/* Misc call by Linux */
 #undef DEBUG_IOCTL_INFO		/* Various debug info */
 #define DEBUG_IOCTL_ERROR	/* What's going wrong */
@@ -549,13 +550,14 @@ typedef u_char		mac_addr[WAVELAN_ADDR_SIZE];	/* Hardware address */
  */
 struct net_local
 {
+  spinlock_t	lock;
   dev_node_t 	node;		/* ???? What is this stuff ???? */
   device *	dev;		/* Reverse link... */
   dev_link_t *	link;		/* pcmcia structure */
   en_stats	stats;		/* Ethernet interface statistics */
   int		nresets;	/* Number of hw resets */
   u_char	configured;	/* If it is configured */
-  u_char	reconfig_82593;	/* Need to reconfigure the controler */
+  u_char	reconfig_82593;	/* Need to reconfigure the controller */
   u_char	promiscuous;	/* Promiscuous mode */
   u_char	allmulticast;	/* All Multicast mode */
   int		mc_count;	/* Number of multicast addresses */
@@ -608,10 +610,6 @@ void wv_roam_cleanup(struct net_device *dev);
 #endif	/* WAVELAN_ROAMING */
 
 /* ----------------------- MISC SUBROUTINES ------------------------ */
-static inline unsigned long	/* flags */
-	wv_splhi(void);		/* Disable interrupts */
-static inline void
-	wv_splx(unsigned long);	/* ReEnable interrupts : flags */
 static void
 	cs_error(client_handle_t, /* Report error to cardmgr */
 		 int,
@@ -671,7 +669,7 @@ static int
 		     char *,
 		     int);
 static inline void
-	wv_82593_reconfig(device *);	/* Reconfigure the controler */
+	wv_82593_reconfig(device *);	/* Reconfigure the controller */
 /* ------------------- DEBUG & INFO SUBROUTINES ------------------- */
 static inline void
 	wv_init_info(device *);	/* display startup info */
@@ -739,11 +737,11 @@ static int
 /**************************** VARIABLES ****************************/
 
 static dev_info_t dev_info = "wavelan_cs";
-static dev_link_t *dev_list = NULL;	/* Linked list of devices */
+static dev_link_t *dev_list;		/* Linked list of devices */
 
 /* WARNING : the following variable MUST be volatile
  * It is used by wv_82593_cmd to syncronise with wavelan_interrupt */ 
-static volatile int	wv_wait_completed = 0;
+static volatile int	wv_wait_completed;
 
 /*
  * Parameters that can be set with 'insmod'
@@ -756,7 +754,7 @@ static int	irq_mask = 0xdeb8;
 static int 	irq_list[4] = { -1 };
 
 /* Shared memory speed, in ns */
-static int	mem_speed = 0;
+static int	mem_speed;
 
 /* New module interface */
 MODULE_PARM(irq_mask, "i");
@@ -765,7 +763,7 @@ MODULE_PARM(mem_speed, "i");
 
 #ifdef WAVELAN_ROAMING		/* Conditional compile, see above in options */
 /* Enable roaming mode ? No ! Please keep this to 0 */
-static int	do_roaming = 0;
+static int	do_roaming;
 MODULE_PARM(do_roaming, "i");
 #endif	/* WAVELAN_ROAMING */
 
