@@ -24,6 +24,9 @@
  *	Edgeport/8i
  *
  * Version history:
+ * 
+ *     (04/08/2001) gb
+ *	- Identify version on module load.
  *
  * 2.0 2001_03_05 greg kroah-hartman
  *	- reworked entire driver to fit properly in with the other usb-serial
@@ -223,6 +226,7 @@
  * 
  */
 
+#include <linux/config.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/signal.h>
@@ -239,21 +243,27 @@
 #include <linux/serial.h>
 #include <linux/ioctl.h>
 #include <linux/proc_fs.h>
+#include <linux/usb.h>
 
 #ifdef CONFIG_USB_SERIAL_DEBUG
-	#define DEBUG
+	static int debug = 1;
 #else
-	#undef DEBUG
+	static int debug;
 #endif
 
-#include <linux/usb.h>
-
 #include "usb-serial.h"
 
 #include "io_edgeport.h"
 #include "io_ionsp.h"		/* info for the iosp messages */
 #include "io_16654.h"		/* 16654 UART defines */
 
+/*
+ * Version Information
+ */
+#define DRIVER_VERSION "v2.0.0"
+#define DRIVER_AUTHOR "Greg Kroah-Hartman <greg@kroah.com> and David Iacovelli"
+#define DRIVER_DESC "Edgeport USB Serial Driver"
+
 /* First, the latest boot code - for first generation edgeports */
 #define IMAGE_ARRAY_NAME	BootCodeImage_GEN1
 #define IMAGE_VERSION_NAME	BootCodeImageVersion_GEN1
@@ -275,10 +285,6 @@
 #include "io_fw_down2.h"	/* Define array OperationalCodeImage[] */
 
 
-/* Module information */
-MODULE_AUTHOR("Greg Kroah-Hartman <greg@kroah.com> and David Iacovelli");
-MODULE_DESCRIPTION("Edgeport USB Serial Driver");
-
 #define MAX_NAME_LEN		64
 
 
@@ -474,47 +480,6 @@ static int  get_string_desc		(struct usb_device *dev, int Id, struct usb_string_
 
 
 
-
-#ifdef DEBUG
-
-/* Dump a buffer in HEX and Ascii */
-void DbgDisplayBuffer( void *pBuffer, __u32 Len )
-{
-	char    DisplayBuf[80];
-	char *  pStr = DisplayBuf;
-	__u8 *pBuf = pBuffer; 
-	__u32   i;
-	__u8   d;
-
-	while (Len) {
-		// Init for new line
-		memset( DisplayBuf, ' ', sizeof( DisplayBuf ));
-		DisplayBuf[79]=0;
-		pStr = DisplayBuf;
-		pStr[54] = '[';
-		pStr[71] = ']';
-
-		for ( i = 0; i < MIN(16, Len) ; i++ ) {
-			d = (__u8)(*pBuf >> 4);
-			pStr[(i*3)+0] = (char)((d < 10) ? d+'0' : d -10 + 'A');
-			d = (__u8)(*pBuf & 0xf);
-			pStr[(i*3)+1] = (char)((d < 10) ? d+'0' : d -10 + 'A');
-
-			if (*pBuf > 31 && *pBuf < 127)
-				pStr[i+55]=*pBuf;
-			else
-				pStr[i+55]='.';
-
-			pBuf++;
-		}
-		Len -= i;
-		dbg("%s", DisplayBuf );
-	}
-}
-#endif
-
-
-
 // ************************************************************************
 // ************************************************************************
 // ************************************************************************
@@ -745,7 +710,6 @@ static void get_product_info(struct edgeport_serial *edge_serial)
 			break;
 	}
 
-#ifdef DEBUG
 	// Dump Product Info structure
 	dbg("**Product Information:");
 	dbg("  ProductId             %x", product_info->ProductId );
@@ -770,7 +734,6 @@ static void get_product_info(struct edgeport_serial *edge_serial)
 	    product_info->ManufactureDescDate[2]+1900);
 	dbg("  iDownloadFile         0x%x",     product_info->iDownloadFile);
 
-#endif
 }
 
 
@@ -812,14 +775,7 @@ static void edge_interrupt_callback (struct urb *urb)
 
 	// process this interrupt-read even if there are no ports open
 	if (length) {
-#ifdef DEBUG
-		int i;
-		printk (KERN_DEBUG __FILE__ ": "__FUNCTION__" - length = %d, data = ", length);
-		for (i = 0; i < length; ++i) {
-			printk ("%.2x ", data[i]);
-		}
-		printk ("\n");
-#endif
+		usb_serial_debug_data (__FILE__, __FUNCTION__, length, data);
 
 		if (length > 1) {
 			bytes_avail = data[0] | (data[1] << 8);
@@ -894,13 +850,7 @@ static void edge_bulk_in_callback (struct urb *urb)
 	if (urb->actual_length) {
 		raw_data_length = urb->actual_length;
 
-#ifdef DEBUG
-		{
-//			int i;
-			dbg (__FUNCTION__" - length = %d, data = ", raw_data_length);
-//			DbgDisplayBuffer((void *)data, raw_data_length);
-		}
-#endif
+		usb_serial_debug_data (__FILE__, __FUNCTION__, raw_data_length, data);
 
 		/* decrement our rxBytes available by the number that we just got */
 		edge_serial->rxBytesAvail -= raw_data_length;
@@ -976,10 +926,8 @@ static void edge_bulk_out_cmd_callback (struct urb *urb)
 
 	dbg(__FUNCTION__);
 
-#ifdef DEBUG	
 	CmdUrbs--;
 	dbg(__FUNCTION__" - FREE URB %p (outstanding %d)", urb, CmdUrbs);
-#endif
 
 
 	/* if this urb had a transfer buffer already (old transfer) free it */
@@ -996,7 +944,7 @@ static void edge_bulk_out_cmd_callback (struct urb *urb)
 	}
 
 	if (status) {
-		dbg(__FUNCTION__" - nonzero write bulk status received: %d", urb->status);
+		dbg(__FUNCTION__" - nonzero write bulk status received: %d", status);
 		return;
 	}
 
@@ -1412,12 +1360,9 @@ static int edge_write (struct usb_serial_port *port, int from_user, const unsign
 		// No need to check for wrap since we can not get to end of fifo in this part
 	}
 
-#ifdef DEBUG
 	if (copySize) {
-		dbg (__FUNCTION__" - length = %d, data = ", copySize);
-		DbgDisplayBuffer((void *)data, copySize);
+		usb_serial_debug_data (__FILE__, __FUNCTION__, copySize, data);
 	}
-#endif
 
 	send_more_port_data((struct edgeport_serial *)port->serial->private, edge_port);
 
@@ -1519,12 +1464,9 @@ static void send_more_port_data(struct edgeport_serial *edge_serial, struct edge
 		fifo->count -= secondhalf;
 	}
 
-#ifdef DEBUG
 	if (count) {
-		dbg (__FUNCTION__" - length = %d, data = ", count);
-		DbgDisplayBuffer((void *)&buffer[2], count);
+		usb_serial_debug_data (__FILE__, __FUNCTION__, count, &buffer[2]);
 	}
-#endif
 
 	/* fill up the urb with all of our data and submit it */
 	FILL_BULK_URB (urb, edge_serial->serial->dev, 
@@ -2498,16 +2440,7 @@ static int write_cmd_usb (struct edgeport_port *edge_port, unsigned char *buffer
 	urb_t *urb;
 	int timeout;
 
-#ifdef DEBUG
-	if (length) {
-		int i;
-		printk (KERN_DEBUG __FILE__ ": "__FUNCTION__" - length = %d, buffer = ", length);
-		for (i = 0; i < length; ++i) {
-			printk ("%.2x ", buffer[i]);
-		}
-		printk ("\n");
-	}
-#endif
+	usb_serial_debug_data (__FILE__, __FUNCTION__, length, buffer);
 
 	/* Allocate our next urb */
 	urb = usb_alloc_urb (0);
@@ -2857,7 +2790,6 @@ static void get_manufacturing_desc (struct edgeport_serial *edge_serial)
 	if (response < 1) {
 		err("error in getting manufacturer descriptor");
 	} else {
-#ifdef DEBUG
 		char string[30];
 		dbg("**Manufacturer Descriptor");
 		dbg("  RomSize:        %dK", edge_serial->manuf_descriptor.RomSize);
@@ -2875,7 +2807,6 @@ static void get_manufacturing_desc (struct edgeport_serial *edge_serial)
 		dbg("  UartType:       %d", edge_serial->manuf_descriptor.UartType);
 		dbg("  IonPid:         %d", edge_serial->manuf_descriptor.IonPid);
 		dbg("  IonConfig:      %d", edge_serial->manuf_descriptor.IonConfig);
-#endif
 	}
 }
 
@@ -3106,7 +3037,7 @@ int __init edgeport_init(void)
 	usb_serial_register (&edgeport_16dual_device);
 	usb_serial_register (&edgeport_compat_id_device);
 	usb_serial_register (&edgeport_8i_device);
-	
+	info(DRIVER_VERSION ":" DRIVER_DESC);
 	return 0;
 }
 
@@ -3139,5 +3070,10 @@ void __exit edgeport_exit (void)
 module_init(edgeport_init);
 module_exit(edgeport_exit);
 
+/* Module information */
+MODULE_AUTHOR( DRIVER_AUTHOR );
+MODULE_DESCRIPTION( DRIVER_DESC );
 
+MODULE_PARM(debug, "i");
+MODULE_PARM_DESC(debug, "Debug enabled or not");
 
