@@ -1,15 +1,16 @@
 /*
   Keyspan USB to Serial Converter driver
  
-  (C) Copyright (C) 2000-2001
-      Hugh Blemings <hugh@misc.nu>
+  (C) Copyright (C) 2000
+      Hugh Blemings <hugh@linuxcare.com>
    
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
-  See http://misc.nu/hugh/keyspan.html for more information.
+  See http://www.linuxcare.com.au/hugh/keyspan.html for more
+  information on this driver.
   
   Code in this driver inspired by and in a number of places taken
   from Brian Warner's original Keyspan-PDA driver.
@@ -23,25 +24,19 @@
   having the patience to sit down and explain why and where he'd changed
   stuff. 
   
-  Tip 'o the hat to IBM (and previously Linuxcare :) for supporting 
-  staff in their work on open source projects.
+  Tip 'o the hat to Linuxcare for supporting staff in their work on
+  open source projects.
 
   Change History
-
-    Mon Oct  8 14:29:00 EST 2001 hugh
-      Fixed bug that prevented mulitport devices operating correctly
-      if they weren't the first unit attached.
-
-    Sat Oct  6 12:31:21 EST 2001 hugh
-      Added support for USA-28XA and -28XB, misc cleanups, break support
-      for usa26 based models thanks to David Gibson.
-
     Thu May 31 11:56:42 PDT 2001 gkh
       switched from using spinlock to a semaphore
    
-    (04/08/2001) gb
+   (04/08/2001) gb
 	Identify version on module load.
    
+   (11/01/2000) Adam J. Richter
+	usb_device_id table support.
+   
     Tue Oct 10 23:15:33 EST 2000 Hugh
       Merged Paul's changes with my USA-49W mods.  Work in progress
       still...
@@ -67,7 +62,7 @@
 #include <linux/errno.h>
 #include <linux/poll.h>
 #include <linux/init.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/fcntl.h>
 #include <linux/tty.h>
 #include <linux/tty_driver.h>
@@ -78,22 +73,18 @@
 
 #ifdef CONFIG_USB_SERIAL_DEBUG
 	static int debug = 1;
-	#define DEBUG
 #else
 	static int debug;
-	#undef DEBUG
 #endif
 
-#include <linux/usb.h>
-
 #include "usb-serial.h"
 #include "keyspan.h"
 
 /*
  * Version Information
  */
-#define DRIVER_VERSION "v1.1.1"
-#define DRIVER_AUTHOR "Hugh Blemings <hugh@misc.nu"
+#define DRIVER_VERSION "v1.0.0"
+#define DRIVER_AUTHOR "Hugh Blemings <hugh@linuxcare.com>"
 #define DRIVER_DESC "Keyspan USB to Serial Converter Driver"
 
 #define INSTAT_BUFLEN	32
@@ -150,7 +141,6 @@ struct keyspan_port_private {
 	int		dsr_state;
 	int		dcd_state;
 	int		ri_state;
-	int		break_on;
 
 	unsigned long	tx_start_time[2];
 	int		resend_cont;	/* need to resend control packet */
@@ -171,17 +161,16 @@ struct keyspan_port_private {
   #define	dbg	printk 
 #endif
 
+static void keyspan_send_setup(struct usb_serial_port *port);
 
 /* Functions used by new usb-serial code. */
-static int __init keyspan_init (void)
+int keyspan_init (void)
 {
 	usb_serial_register (&keyspan_usa18x_pre_device);
 	usb_serial_register (&keyspan_usa19_pre_device);
 	usb_serial_register (&keyspan_usa19w_pre_device);
 	usb_serial_register (&keyspan_usa28_pre_device);
 	usb_serial_register (&keyspan_usa28x_pre_device);
-	usb_serial_register (&keyspan_usa28xa_pre_device);
-	usb_serial_register (&keyspan_usa28xb_pre_device);
 	usb_serial_register (&keyspan_usa49w_pre_device);
 
 	usb_serial_register (&keyspan_usa18x_device);
@@ -189,8 +178,6 @@ static int __init keyspan_init (void)
 	usb_serial_register (&keyspan_usa19w_device);
 	usb_serial_register (&keyspan_usa28_device);
 	usb_serial_register (&keyspan_usa28x_device);
-	usb_serial_register (&keyspan_usa28xa_device);
-	/* We don't need a separate entry for the usa28xb as it appears as a 28x anyway */
 	usb_serial_register (&keyspan_usa49w_device);
 
 	info(DRIVER_VERSION ":" DRIVER_DESC);
@@ -198,15 +185,13 @@ static int __init keyspan_init (void)
 	return 0;
 }
 
-static void __exit keyspan_exit (void)
+void keyspan_exit (void)
 {
 	usb_serial_deregister (&keyspan_usa18x_pre_device);
 	usb_serial_deregister (&keyspan_usa19_pre_device);
 	usb_serial_deregister (&keyspan_usa19w_pre_device);
 	usb_serial_deregister (&keyspan_usa28_pre_device);
 	usb_serial_deregister (&keyspan_usa28x_pre_device);
-	usb_serial_deregister (&keyspan_usa28xa_pre_device);
-	usb_serial_deregister (&keyspan_usa28xb_pre_device);
 	usb_serial_deregister (&keyspan_usa49w_pre_device);
 
 	usb_serial_deregister (&keyspan_usa18x_device);
@@ -214,8 +199,6 @@ static void __exit keyspan_exit (void)
 	usb_serial_deregister (&keyspan_usa19w_device);
 	usb_serial_deregister (&keyspan_usa28_device);
 	usb_serial_deregister (&keyspan_usa28x_device);
-	usb_serial_deregister (&keyspan_usa28xa_device);
-	/* We don't need a separate entry for the usa28xb as it appears as a 28x anyway */
 	usb_serial_deregister (&keyspan_usa49w_device);
 }
 
@@ -224,30 +207,19 @@ module_exit(keyspan_exit);
 
 static void keyspan_rx_throttle (struct usb_serial_port *port)
 {
-	dbg("keyspan_rx_throttle port %d\n", port->number);
+	dbg("keyspan_rx_throttle port %d", port->number);
 }
 
 
 static void keyspan_rx_unthrottle (struct usb_serial_port *port)
 {
-	dbg("keyspan_rx_unthrottle port %d\n", port->number);
+	dbg("keyspan_rx_unthrottle port %d", port->number);
 }
 
 
 static void keyspan_break_ctl (struct usb_serial_port *port, int break_state)
 {
-	struct keyspan_port_private 	*p_priv;
-
- 	dbg("keyspan_break_ctl\n");
-
-	p_priv = (struct keyspan_port_private *)port->private;
-
-	if (break_state == -1)
-		p_priv->break_on = 1;
-	else
-		p_priv->break_on = 0;
-
-	keyspan_send_setup(port, 0);
+	dbg("keyspan_break_ctl");
 }
 
 
@@ -259,7 +231,7 @@ static void keyspan_set_termios (struct usb_serial_port *port,
 	const keyspan_device_details	*d_details;
 	unsigned int 			cflag;
 
-	dbg(__FUNCTION__ ".\n"); 
+	/*  dbg(__FUNCTION__ "."); */
 
 	p_priv = (struct keyspan_port_private *)(port->private);
 	d_details = p_priv->device_details;
@@ -280,7 +252,7 @@ static void keyspan_set_termios (struct usb_serial_port *port,
 	p_priv->cflag = cflag;
 	p_priv->flow_control = (cflag & CRTSCTS)? flow_cts: flow_none;
 
-	keyspan_send_setup(port, 0);
+	keyspan_send_setup(port);
 }
 
 static int keyspan_ioctl(struct usb_serial_port *port, struct file *file,
@@ -309,7 +281,7 @@ static int keyspan_ioctl(struct usb_serial_port *port, struct file *file,
 			return -EFAULT;
 		p_priv->rts_state = ((value & TIOCM_RTS) ? 1 : 0);
 		p_priv->dtr_state = ((value & TIOCM_DTR) ? 1 : 0);
-		keyspan_send_setup(port, 0);
+		keyspan_send_setup(port);
 		return 0;
 
 	case TIOCMBIS:
@@ -321,7 +293,7 @@ static int keyspan_ioctl(struct usb_serial_port *port, struct file *file,
 			p_priv->rts_state = set;
 		if (value & TIOCM_DTR)
 			p_priv->dtr_state = set;
-		keyspan_send_setup(port, 0);
+		keyspan_send_setup(port);
 		return 0;
 	}
 
@@ -336,15 +308,15 @@ static int keyspan_write(struct usb_serial_port *port, int from_user,
 	struct keyspan_port_private 	*p_priv;
 	const keyspan_device_details	*d_details;
 	int				flip;
-	int 				left, todo;
-	urb_t 				*this_urb;
-	int 				err;
+	int left, todo;
+	urb_t *this_urb;
+	int err;
 
 	p_priv = (struct keyspan_port_private *)(port->private);
 	d_details = p_priv->device_details;
 
 #if 0
-	dbg(__FUNCTION__ " for port %d (%d chars [%x]), flip=%d\n",
+	dbg(__FUNCTION__ " for port %d (%d chars [%x]), flip=%d",
 	    port->number, count, buf[0], p_priv->out_flip);
 #endif
 
@@ -358,12 +330,10 @@ static int keyspan_write(struct usb_serial_port *port, int from_user,
 		/* Check we have a valid urb/endpoint before we use it... */
 		if ((this_urb = p_priv->out_urbs[flip]) == 0) {
 			/* no bulk out, so return 0 bytes written */
-			dbg(__FUNCTION__ " no output urb :(\n");
+			dbg(__FUNCTION__ " no output urb :(");
 			return count;
 		}
 
-		dbg(__FUNCTION__ " endpoint %d\n", usb_pipeendpoint(this_urb->pipe));
-
 		if (this_urb->status == -EINPROGRESS) {
 			if (this_urb->transfer_flags & USB_ASYNC_UNLINK)
 				break;
@@ -379,8 +349,7 @@ static int keyspan_write(struct usb_serial_port *port, int from_user,
 		((char *)this_urb->transfer_buffer)[0] = 0;
 
 		if (from_user) {
-			if (copy_from_user(this_urb->transfer_buffer + 1, buf, todo))
-				return -EFAULT;
+			copy_from_user(this_urb->transfer_buffer + 1, buf, todo);
 		} else {
 			memcpy (this_urb->transfer_buffer + 1, buf, todo);
 		}
@@ -392,7 +361,7 @@ static int keyspan_write(struct usb_serial_port *port, int from_user,
 		this_urb->transfer_flags &= ~USB_ASYNC_UNLINK;
 		this_urb->dev = port->serial->dev;
 		if ((err = usb_submit_urb(this_urb)) != 0) {
-			dbg("usb_submit_urb(write bulk) failed (%d)\n", err);
+			dbg("usb_submit_urb(write bulk) failed (%d)", err);
 		}
 		p_priv->tx_start_time[flip] = jiffies;
 
@@ -412,12 +381,12 @@ static void	usa26_indat_callback(struct urb *urb)
 	struct tty_struct	*tty;
 	unsigned char 		*data = urb->transfer_buffer;
 
-	dbg ("%s\n", __FUNCTION__); 
+	/*  dbg (__FUNCTION__); */
 
 	endpoint = usb_pipeendpoint(urb->pipe);
 
 	if (urb->status) {
-		dbg(__FUNCTION__ "nonzero status: %x on endpoint %d.\n",
+		dbg(__FUNCTION__ "nonzero status: %x on endpoint %d.",
 			      		urb->status, endpoint);
 		return;
 	}
@@ -450,7 +419,7 @@ static void	usa26_indat_callback(struct urb *urb)
 		/* Resubmit urb so we continue receiving */
 	urb->dev = port->serial->dev;
 	if ((err = usb_submit_urb(urb)) != 0) {
-		dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n", err);
+		dbg(__FUNCTION__ "resubmit read urb failed. (%d)", err);
 	}
 	return;
 }
@@ -463,7 +432,7 @@ static void	usa2x_outdat_callback(struct urb *urb)
 
 	port = (struct usb_serial_port *) urb->context;
 	p_priv = (struct keyspan_port_private *)(port->private);
-	dbg (__FUNCTION__ " urb %d\n", urb == p_priv->out_urbs[1]); 
+	/*  dbg (__FUNCTION__ " urb %d", urb == p_priv->out_urbs[1]); */
 
 	if (port->active) {
 		queue_task(&port->tqueue, &tq_immediate);
@@ -473,7 +442,7 @@ static void	usa2x_outdat_callback(struct urb *urb)
 
 static void	usa26_inack_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__); 
+	dbg (__FUNCTION__);
 	
 }
 
@@ -486,8 +455,8 @@ static void	usa26_outcont_callback(struct urb *urb)
 	p_priv = (struct keyspan_port_private *)(port->private);
 
 	if (p_priv->resend_cont) {
-		dbg (__FUNCTION__ " sending setup\n"); 
-		keyspan_usa26_send_setup(port->serial, port, 0);
+		/*  dbg (__FUNCTION__ " sending setup"); */
+		keyspan_usa26_send_setup(port->serial, port);
 	}
 }
 
@@ -503,18 +472,18 @@ static void	usa26_instat_callback(struct urb *urb)
 	serial = (struct usb_serial *) urb->context;
 
 	if (urb->status) {
-		dbg(__FUNCTION__ " nonzero status: %x\n", urb->status);
+		dbg(__FUNCTION__ " nonzero status: %x", urb->status);
 		return;
 	}
 	if (urb->actual_length != 9) {
-		dbg(__FUNCTION__ " %d byte report??\n", urb->actual_length);
+		dbg(__FUNCTION__ " %d byte report??", urb->actual_length);
 		goto exit;
 	}
 
 	msg = (keyspan_usa26_portStatusMessage *)data;
 
 #if 0
-	dbg(__FUNCTION__ " port status: port %d cts %d dcd %d dsr %d ri %d toff %d txoff %d rxen %d cr %d\n",
+	dbg(__FUNCTION__ " port status: port %d cts %d dcd %d dsr %d ri %d toff %d txoff %d rxen %d cr %d",
 	    msg->port, msg->hskia_cts, msg->gpia_dcd, msg->dsr, msg->ri, msg->_txOff,
 	    msg->_txXoff, msg->rxEnabled, msg->controlResponse);
 #endif
@@ -524,7 +493,7 @@ static void	usa26_instat_callback(struct urb *urb)
 
 	/* Check port number from message and retrieve private data */	
 	if (msg->port >= serial->num_ports) {
-		dbg ("Unexpected port number %d\n", msg->port);
+		dbg ("Unexpected port number %d", msg->port);
 		goto exit;
 	}
 	port = &serial->port[msg->port];
@@ -549,13 +518,13 @@ exit:
 	/* Resubmit urb so we continue receiving */
 	urb->dev = serial->dev;
 	if ((err = usb_submit_urb(urb)) != 0) {
-		dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n", err);
+		dbg(__FUNCTION__ "resubmit read urb failed. (%d)", err);
 	}
 }
 
 static void	usa26_glocont_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__);
+	dbg (__FUNCTION__);
 	
 }
 
@@ -568,7 +537,7 @@ static void     usa28_indat_callback(struct urb *urb)
 	unsigned char           *data;
 	struct keyspan_port_private             *p_priv;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	port = (struct usb_serial_port *) urb->context;
 	p_priv = (struct keyspan_port_private *)(port->private);
@@ -580,7 +549,7 @@ static void     usa28_indat_callback(struct urb *urb)
 	do {
 		if (urb->status) {
 			dbg(__FUNCTION__ "nonzero status: %x on endpoint
-%d.\n",
+%d.",
 			    urb->status, usb_pipeendpoint(urb->pipe));
 			return;
 		}
@@ -600,7 +569,7 @@ static void     usa28_indat_callback(struct urb *urb)
 		/* Resubmit urb so we continue receiving */
 		urb->dev = port->serial->dev;
 		if ((err = usb_submit_urb(urb)) != 0) {
-			dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n",
+			dbg(__FUNCTION__ "resubmit read urb failed. (%d)",
 err);
 		}
 		p_priv->in_flip ^= 1;
@@ -611,7 +580,7 @@ err);
 
 static void	usa28_inack_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__);
+	dbg (__FUNCTION__);
 }
 
 static void	usa28_outcont_callback(struct urb *urb)
@@ -623,8 +592,8 @@ static void	usa28_outcont_callback(struct urb *urb)
 	p_priv = (struct keyspan_port_private *)(port->private);
 
 	if (p_priv->resend_cont) {
-		dbg (__FUNCTION__ " sending setup\n");
-		keyspan_usa28_send_setup(port->serial, port, 0);
+		dbg (__FUNCTION__ " sending setup");
+		keyspan_usa28_send_setup(port->serial, port);
 	}
 }
 
@@ -641,16 +610,16 @@ static void	usa28_instat_callback(struct urb *urb)
 	serial = (struct usb_serial *) urb->context;
 
 	if (urb->status) {
-		dbg(__FUNCTION__ " nonzero status: %x\n", urb->status);
+		dbg(__FUNCTION__ " nonzero status: %x", urb->status);
 		return;
 	}
 
 	if (urb->actual_length != sizeof(struct keyspan_usa28_portStatusMessage)) {
-		dbg(__FUNCTION__ " bad length %d\n", urb->actual_length);
+		dbg(__FUNCTION__ " bad length %d", urb->actual_length);
 		goto exit;
 	}
 
-	/*dbg(__FUNCTION__ " %x %x %x %x %x %x %x %x %x %x %x %x\n",
+	/*dbg(__FUNCTION__ " %x %x %x %x %x %x %x %x %x %x %x %x",
 	    data[0], data[1], data[2], data[3], data[4], data[5],
 	    data[6], data[7], data[8], data[9], data[10], data[11]);*/
 	
@@ -660,7 +629,7 @@ static void	usa28_instat_callback(struct urb *urb)
 
 		/* Check port number from message and retrieve private data */	
 	if (msg->port >= serial->num_ports) {
-		dbg ("Unexpected port number %d\n", msg->port);
+		dbg ("Unexpected port number %d", msg->port);
 		goto exit;
 	}
 	port = &serial->port[msg->port];
@@ -685,13 +654,13 @@ exit:
 		/* Resubmit urb so we continue receiving */
 	urb->dev = serial->dev;
 	if ((err = usb_submit_urb(urb)) != 0) {
-		dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n", err);
+		dbg(__FUNCTION__ "resubmit read urb failed. (%d)", err);
 	}
 }
 
 static void	usa28_glocont_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__);
+	dbg (__FUNCTION__);
 }
 
 
@@ -702,7 +671,7 @@ static void	usa49_glocont_callback(struct urb *urb)
 	struct keyspan_port_private *p_priv;
 	int i;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	serial = (struct usb_serial *) urb->context;
 	for (i = 0; i < serial->num_ports; ++i) {
@@ -710,8 +679,8 @@ static void	usa49_glocont_callback(struct urb *urb)
 		p_priv = (struct keyspan_port_private *)(port->private);
 
 		if (p_priv->resend_cont) {
-			dbg (__FUNCTION__ " sending setup\n"); 
-			keyspan_usa49_send_setup(serial, port, 0);
+			/*  dbg (__FUNCTION__ " sending setup"); */
+			keyspan_usa49_send_setup(serial, port);
 			break;
 		}
 	}
@@ -729,21 +698,21 @@ static void	usa49_instat_callback(struct urb *urb)
 	struct keyspan_port_private	 	*p_priv;
 	int old_dcd_state;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	serial = (struct usb_serial *) urb->context;
 
 	if (urb->status) {
-		dbg(__FUNCTION__ " nonzero status: %x\n", urb->status);
+		dbg(__FUNCTION__ " nonzero status: %x", urb->status);
 		return;
 	}
 
 	if (urb->actual_length != sizeof(struct keyspan_usa49_portStatusMessage)) {
-		dbg(__FUNCTION__ " bad length %d\n", urb->actual_length);
+		dbg(__FUNCTION__ " bad length %d", urb->actual_length);
 		goto exit;
 	}
 
-	/*dbg(__FUNCTION__ " %x %x %x %x %x %x %x %x %x %x %x\n",
+	/*dbg(__FUNCTION__ " %x %x %x %x %x %x %x %x %x %x %x",
 	    data[0], data[1], data[2], data[3], data[4], data[5],
 	    data[6], data[7], data[8], data[9], data[10]);*/
 	
@@ -752,7 +721,7 @@ static void	usa49_instat_callback(struct urb *urb)
 
 		/* Check port number from message and retrieve private data */	
 	if (msg->portNumber >= serial->num_ports) {
-		dbg ("Unexpected port number %d\n", msg->portNumber);
+		dbg ("Unexpected port number %d", msg->portNumber);
 		goto exit;
 	}
 	port = &serial->port[msg->portNumber];
@@ -778,13 +747,13 @@ exit:
 	urb->dev = serial->dev;
 
 	if ((err = usb_submit_urb(urb)) != 0) {
-		dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n", err);
+		dbg(__FUNCTION__ "resubmit read urb failed. (%d)", err);
 	}
 }
 
 static void	usa49_inack_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__);
+	dbg (__FUNCTION__);
 }
 
 static void	usa49_indat_callback(struct urb *urb)
@@ -795,12 +764,12 @@ static void	usa49_indat_callback(struct urb *urb)
 	struct tty_struct	*tty;
 	unsigned char 		*data = urb->transfer_buffer;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	endpoint = usb_pipeendpoint(urb->pipe);
 
 	if (urb->status) {
-		dbg(__FUNCTION__ "nonzero status: %x on endpoint %d.\n",
+		dbg(__FUNCTION__ "nonzero status: %x on endpoint %d.",
 			      		urb->status, endpoint);
 		return;
 	}
@@ -833,21 +802,21 @@ static void	usa49_indat_callback(struct urb *urb)
 		/* Resubmit urb so we continue receiving */
 	urb->dev = port->serial->dev;
 	if ((err = usb_submit_urb(urb)) != 0) {
-		dbg(__FUNCTION__ "resubmit read urb failed. (%d)\n", err);
+		dbg(__FUNCTION__ "resubmit read urb failed. (%d)", err);
 	}
 }
 
 /* not used, usa-49 doesn't have per-port control endpoints */
 static void	usa49_outcont_callback(struct urb *urb)
 {
-	dbg ("%s\n", __FUNCTION__);
+	dbg (__FUNCTION__);
 }
 
 
 
 static int keyspan_write_room (struct usb_serial_port *port)
 {
-	dbg("keyspan_write_room called\n");
+//	dbg("keyspan_write_room called");
 	return (32);
 
 }
@@ -872,8 +841,7 @@ static int keyspan_open (struct usb_serial_port *port, struct file *filp)
 	p_priv = (struct keyspan_port_private *)(port->private);
 	d_details = s_priv->device_details;
 	
-	dbg("keyspan_open called for port%d.\n", port->number); 
-
+	/*  dbg("keyspan_open called."); */
 	MOD_INC_USE_COUNT;
 
 	down (&port->sem);
@@ -886,7 +854,7 @@ static int keyspan_open (struct usb_serial_port *port, struct file *filp)
 		return 0;
 
 	p_priv = (struct keyspan_port_private *)(port->private);
-	
+
 	/* Set some sane defaults */
 	p_priv->rts_state = 1;
 	p_priv->dtr_state = 1;
@@ -897,7 +865,7 @@ static int keyspan_open (struct usb_serial_port *port, struct file *filp)
 			continue;
 		urb->dev = serial->dev;
 		if ((err = usb_submit_urb(urb)) != 0) {
-			dbg(__FUNCTION__ " submit urb %d failed (%d)\n", i, err);
+			dbg(__FUNCTION__ " submit urb %d failed (%d)", i, err);
 		}
 	}
 
@@ -917,44 +885,30 @@ static inline void stop_urb(urb_t *urb)
 static void keyspan_close(struct usb_serial_port *port, struct file *filp)
 {
 	int			i;
-	struct usb_serial	*serial;
+	struct usb_serial	*serial = port->serial; /* FIXME should so sanity check */
 	struct keyspan_serial_private 	*s_priv;
 	struct keyspan_port_private 	*p_priv;
 
-	serial = get_usb_serial (port, __FUNCTION__);
-	if (!serial)
-		return;
-
-	dbg("keyspan_close called\n");
+	/*  dbg("keyspan_close called"); */
 	s_priv = (struct keyspan_serial_private *)(serial->private);
 	p_priv = (struct keyspan_port_private *)(port->private);
-	
-	p_priv->rts_state = 0;
-	p_priv->dtr_state = 0;
-	
-	if (serial->dev)
-		keyspan_send_setup(port, 1);
-
-	/*while (p_priv->outcont_urb->status == -EINPROGRESS) {
-		dbg("close - urb in progress\n");
-	}*/
-
-	p_priv->out_flip = 0;
-	p_priv->in_flip = 0;
 
 	down (&port->sem);
 
 	if (--port->open_count <= 0) {
 		if (port->active) {
-			if (serial->dev) {
-				/* Stop reading/writing urbs */
-				stop_urb(p_priv->inack_urb);
-				stop_urb(p_priv->outcont_urb);
-				for (i = 0; i < 2; i++) {
-					stop_urb(p_priv->in_urbs[i]);
-					stop_urb(p_priv->out_urbs[i]);
-				}
+			/* Stop reading/writing urbs */
+			stop_urb(p_priv->inack_urb);
+			stop_urb(p_priv->outcont_urb);
+			for (i = 0; i < 2; i++) {
+				stop_urb(p_priv->in_urbs[i]);
+				stop_urb(p_priv->out_urbs[i]);
 			}
+			/* Now done in shutdown 
+			if (atomic_dec_return(&s_priv->active_count) <= 0) {
+				stop_urb(s_priv->instat_urb);
+				stop_urb(s_priv->glocont_urb);
+			} */
 		}
 		port->active = 0;
 		port->open_count = 0;
@@ -973,53 +927,43 @@ static int keyspan_fake_startup (struct usb_serial *serial)
 	const struct ezusb_hex_record 	*record;
 	char				*fw_name;
 
-	dbg("Keyspan startup version %04x product %04x\n",
+	dbg("Keyspan startup version %04x product %04x",
 	    serial->dev->descriptor.bcdDevice,
 	    serial->dev->descriptor.idProduct); 
 	
 	if ((serial->dev->descriptor.bcdDevice & 0x8000) != 0x8000) {
-		dbg("Firmware already loaded.  Quitting.\n");
+		dbg("Firmware already loaded.  Quitting.");
 		return(1);
 	}
 
 		/* Select firmware image on the basis of idProduct */
 	switch (serial->dev->descriptor.idProduct) {
-	case KEYSPAN_USA28_PRE_PRODUCT_ID:
+	case 0x0101:
 		record = &keyspan_usa28_firmware[0];
 		fw_name = "USA28";
 		break;
 
-	case KEYSPAN_USA28X_PRE_PRODUCT_ID:
+	case 0x0102:
 		record = &keyspan_usa28x_firmware[0];
 		fw_name = "USA28X";
 		break;
 
-	case KEYSPAN_USA28XA_PRE_PRODUCT_ID:
-		record = &keyspan_usa28xa_firmware[0];
-		fw_name = "USA28XA";
-		break;
-
-	case KEYSPAN_USA28XB_PRE_PRODUCT_ID:
-		record = &keyspan_usa28xb_firmware[0];
-		fw_name = "USA28XB";
-		break;
-
-	case KEYSPAN_USA19_PRE_PRODUCT_ID:
+	case 0x0103:
 		record = &keyspan_usa19_firmware[0];
 		fw_name = "USA19";
 		break;
 			     
-	case KEYSPAN_USA18X_PRE_PRODUCT_ID:
+	case 0x0105:
 		record = &keyspan_usa18x_firmware[0];
 		fw_name = "USA18X";
 		break;
 			     
-	case KEYSPAN_USA19W_PRE_PRODUCT_ID:
+	case 0x0106:
 		record = &keyspan_usa19w_firmware[0];
 		fw_name = "USA19W";
 		break;
 		
-	case KEYSPAN_USA49W_PRE_PRODUCT_ID:
+	case 0x0109:
 		record = &keyspan_usa49w_firmware[0];
 		fw_name = "USA49W";
 		break;
@@ -1035,7 +979,7 @@ static int keyspan_fake_startup (struct usb_serial *serial)
 		return(1);
 	}
 
-	dbg("Uploading Keyspan %s firmware.\n", fw_name);
+	dbg("Uploading Keyspan %s firmware.", fw_name);
 
 		/* download the firmware image */
 	response = ezusb_set_reset(serial, 1);
@@ -1071,10 +1015,10 @@ static urb_t *keyspan_setup_urb(struct usb_serial *serial, int endpoint,
 	if (endpoint == -1)
 		return NULL;		/* endpoint not needed */
 
-	dbg (__FUNCTION__ " alloc for endpoint %d.\n", endpoint);
+	/*  dbg (__FUNCTION__ " alloc for endpoint %d.", endpoint); */
 	urb = usb_alloc_urb(0);		/* No ISO */
 	if (urb == NULL) {
-		dbg (__FUNCTION__ " alloc for endpoint %d failed.\n", endpoint);
+		dbg (__FUNCTION__ " alloc for endpoint %d failed.", endpoint);
 		return NULL;
 	}
 
@@ -1086,7 +1030,7 @@ static urb_t *keyspan_setup_urb(struct usb_serial *serial, int endpoint,
 	return urb;
 }
 
-static struct callbacks {
+struct callbacks {
 	void	(*instat_callback)(urb_t *);
 	void	(*glocont_callback)(urb_t *);
 	void	(*indat_callback)(urb_t *);
@@ -1133,7 +1077,7 @@ static void keyspan_setup_urbs(struct usb_serial *serial)
 	struct callbacks		*cback;
 	int				endp;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	s_priv = (struct keyspan_serial_private *)(serial->private);
 	d_details = s_priv->device_details;
@@ -1252,7 +1196,7 @@ static int keyspan_usa19w_calc_baud(u32 baud_rate, u32 baudclk,
 	u8	best_prescaler;
 	int	i;
 
-	dbg (__FUNCTION__ " %d.\n", baud_rate);
+	/*  dbg (__FUNCTION__ " %d.", baud_rate); */
 
 		/* prevent divide by zero */
 	if( (b16 = baud_rate * 16L) == 0) {
@@ -1308,8 +1252,7 @@ static int keyspan_usa19w_calc_baud(u32 baud_rate, u32 baudclk,
 }
 
 static int keyspan_usa26_send_setup(struct usb_serial *serial,
-				    struct usb_serial_port *port,
-				    int reset_port)
+				    struct usb_serial_port *port)
 {
 	struct keyspan_usa26_portControlMessage	msg;		
 	struct keyspan_serial_private 		*s_priv;
@@ -1319,7 +1262,7 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 	urb_t *this_urb;
 	int err;
 
-	dbg ("%s reset=%d\n", __FUNCTION__, reset_port); 
+	/*  dbg (__FUNCTION__); */
 
 	s_priv = (struct keyspan_serial_private *)(serial->private);
 	p_priv = (struct keyspan_port_private *)(port->private);
@@ -1328,11 +1271,9 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 	outcont_urb = d_details->outcont_endpoints[port->number];
 	this_urb = p_priv->outcont_urb;
 
-	dbg(__FUNCTION__ " endpoint %d\n", usb_pipeendpoint(this_urb->pipe));
-
 		/* Make sure we have an urb then send the message */
 	if (this_urb == NULL) {
-		dbg(__FUNCTION__ " oops no urb.\n");
+		dbg(__FUNCTION__ " oops no urb.");
 		return -1;
 	}
 
@@ -1351,7 +1292,7 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 		if (d_details->calculate_baud_rate
 		    (p_priv->baud, d_details->baudclk, &msg.baudHi,
 		     &msg.baudLo, &msg.prescaler) == KEYSPAN_INVALID_BAUD_RATE ) {
-			dbg(__FUNCTION__ "Invalid baud rate %d requested, using 9600.\n",
+			dbg(__FUNCTION__ "Invalid baud rate %d requested, using 9600.",
 			    p_priv->baud);
 			msg.baudLo = 0;
 			msg.baudHi = 125;	/* Values for 9600 baud */
@@ -1389,31 +1330,17 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 	msg.forwardingLength = 1;
 	msg.xonChar = 17;
 	msg.xoffChar = 19;
-
-	if (reset_port) {
-		msg._txOn = 0;
-		msg._txOff = 1;
-		msg.txFlush = 0;
-		msg.txBreak = 0;
-		msg.rxOn = 0;
-		msg.rxOff = 1;
-		msg.rxFlush = 1;
-		msg.rxForward = 0;
-		msg.returnStatus = 0;
-		msg.resetDataToggle = 0xff;
-	}
-	else {
-		msg._txOn = (! p_priv->break_on);
-		msg._txOff = 0;
-		msg.txFlush = 0;
-		msg.txBreak = (p_priv->break_on);
-		msg.rxOn = 1;
-		msg.rxOff = 0;
-		msg.rxFlush = 0;
-		msg.rxForward = 0;
-		msg.returnStatus = 0;
-		msg.resetDataToggle = 0x0;
-	}
+	
+	msg._txOn = 1;
+	msg._txOff = 0;
+	msg.txFlush = 0;
+	msg.txBreak = 0;
+	msg.rxOn = 1;
+	msg.rxOff = 0;
+	msg.rxFlush = 0;
+	msg.rxForward = 0;
+	/*msg.returnStatus = 1;
+	msg.resetDataToggle = 0xff;*/
 
 		/* Do handshaking outputs */	
 	msg.setTxTriState_setRts = 0xff;
@@ -1430,7 +1357,7 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 
 	this_urb->dev = serial->dev;
 	if ((err = usb_submit_urb(this_urb)) != 0) {
-		dbg(__FUNCTION__ " usb_submit_urb(setup) failed (%d)\n", err);
+		dbg(__FUNCTION__ " usb_submit_urb(setup) failed (%d)", err);
 	}
 #if 0
 	else {
@@ -1444,8 +1371,7 @@ static int keyspan_usa26_send_setup(struct usb_serial *serial,
 }
 
 static int keyspan_usa28_send_setup(struct usb_serial *serial,
-				    struct usb_serial_port *port,
-				    int reset_port)
+				    struct usb_serial_port *port)
 {
 	struct keyspan_usa28_portControlMessage	msg;		
 	struct keyspan_serial_private	 	*s_priv;
@@ -1460,13 +1386,13 @@ static int keyspan_usa28_send_setup(struct usb_serial *serial,
 
 	/* only do something if we have a bulk out endpoint */
 	if ((this_urb = p_priv->outcont_urb) == NULL) {
-		dbg(__FUNCTION__ " oops no urb.\n");
+		dbg(__FUNCTION__ " oops no urb.");
 		return -1;
 	}
 
 	p_priv->resend_cont = 1;
 	if (this_urb->status == -EINPROGRESS) {
-		dbg (__FUNCTION__ " already writing\n");
+		dbg (__FUNCTION__ " already writing");
 		return(-1);
 	}
 
@@ -1475,7 +1401,7 @@ static int keyspan_usa28_send_setup(struct usb_serial *serial,
 	msg.setBaudRate = 1;
 	if (keyspan_usa19_calc_baud(p_priv->baud, d_details->baudclk,
 		&msg.baudHi, &msg.baudLo, NULL) == KEYSPAN_INVALID_BAUD_RATE ) {
-		dbg(__FUNCTION__ "Invalid baud rate requested %d.", p_priv->baud);
+		dbg(__FUNCTION__ "Invalid baud rate requested %d.", 9600);
 		msg.baudLo = 0xff;
 		msg.baudHi = 0xb2;	/* Values for 9600 baud */
 	}
@@ -1516,7 +1442,7 @@ static int keyspan_usa28_send_setup(struct usb_serial *serial,
 
 	this_urb->dev = serial->dev;
 	if ((err = usb_submit_urb(this_urb)) != 0) {
-		dbg(__FUNCTION__ " usb_submit_urb(setup) failed\n");
+		dbg(__FUNCTION__ " usb_submit_urb(setup) failed");
 	}
 #if 0
 	else {
@@ -1529,19 +1455,17 @@ static int keyspan_usa28_send_setup(struct usb_serial *serial,
 }
 
 static int keyspan_usa49_send_setup(struct usb_serial *serial,
-				    struct usb_serial_port *port,
-				    int reset_port)
+				    struct usb_serial_port *port)
 {
 	struct keyspan_usa49_portControlMessage	msg;		
 	struct keyspan_serial_private 		*s_priv;
 	struct keyspan_port_private 		*p_priv;
 	const  keyspan_device_details		*d_details;
 	int 					glocont_urb;
-	urb_t 					*this_urb;
-	int 					err;
-	int					device_port;
+	urb_t *this_urb;
+	int err;
 
-	dbg ("%s\n", __FUNCTION__);
+	/*  dbg (__FUNCTION__); */
 
 	s_priv = (struct keyspan_serial_private *)(serial->private);
 	p_priv = (struct keyspan_port_private *)(port->private);
@@ -1550,14 +1474,11 @@ static int keyspan_usa49_send_setup(struct usb_serial *serial,
 	glocont_urb = d_details->glocont_endpoint;
 	this_urb = s_priv->glocont_urb;
 
-		/* Work out which port within the device is being setup */
-	device_port = port->number - port->serial->minor;
-
-	dbg(__FUNCTION__ " endpoint %d port %d (%d)\n", usb_pipeendpoint(this_urb->pipe), port->number, device_port);
+	/*  dbg(__FUNCTION__ " port %d\n", port->number); */
 
 		/* Make sure we have an urb then send the message */
 	if (this_urb == NULL) {
-		dbg(__FUNCTION__ " oops no urb for port %d.\n", port->number);
+		dbg(__FUNCTION__ " oops no urb for port %d.", port->number);
 		return -1;
 	}
 
@@ -1569,8 +1490,7 @@ static int keyspan_usa49_send_setup(struct usb_serial *serial,
 
 	memset(&msg, 0, sizeof (struct keyspan_usa49_portControlMessage));
 
-	/*msg.portNumber = port->number;*/
-	msg.portNumber = device_port;
+	msg.portNumber = port->number;
 	
 		/* Only set baud rate if it's changed */	
 	if (p_priv->old_baud != p_priv->baud) {
@@ -1579,7 +1499,7 @@ static int keyspan_usa49_send_setup(struct usb_serial *serial,
 		if (d_details->calculate_baud_rate
 		    (p_priv->baud, d_details->baudclk, &msg.baudHi,
 		     &msg.baudLo, &msg.prescaler) == KEYSPAN_INVALID_BAUD_RATE ) {
-			dbg(__FUNCTION__ "Invalid baud rate %d requested, using 9600.\n",
+			dbg(__FUNCTION__ "Invalid baud rate %d requested, using 9600.",
 			    p_priv->baud);
 			msg.baudLo = 0;
 			msg.baudHi = 125;	/* Values for 9600 baud */
@@ -1644,7 +1564,7 @@ static int keyspan_usa49_send_setup(struct usb_serial *serial,
 
 	this_urb->dev = serial->dev;
 	if ((err = usb_submit_urb(this_urb)) != 0) {
-		dbg(__FUNCTION__ " usb_submit_urb(setup) failed (%d)\n", err);
+		dbg(__FUNCTION__ " usb_submit_urb(setup) failed (%d)", err);
 	}
 #if 0
 	else {
@@ -1657,7 +1577,7 @@ static int keyspan_usa49_send_setup(struct usb_serial *serial,
 	return (0);
 }
 
-static void keyspan_send_setup(struct usb_serial_port *port, int reset_port)
+static void keyspan_send_setup(struct usb_serial_port *port)
 {
 	struct usb_serial *serial = port->serial;
 	struct keyspan_serial_private 	*s_priv;
@@ -1668,13 +1588,13 @@ static void keyspan_send_setup(struct usb_serial_port *port, int reset_port)
 
 	switch (d_details->msg_format) {
 	case msg_usa26:
-		keyspan_usa26_send_setup(serial, port, reset_port);
+		keyspan_usa26_send_setup(serial, port);
 		break;
 	case msg_usa28:
-		keyspan_usa28_send_setup(serial, port, reset_port);
+		keyspan_usa28_send_setup(serial, port);
 		break;
 	case msg_usa49:
-		keyspan_usa49_send_setup(serial, port, reset_port);
+		keyspan_usa49_send_setup(serial, port);
 		break;
 	}
 }
@@ -1689,7 +1609,7 @@ static int keyspan_startup (struct usb_serial *serial)
 	struct keyspan_port_private	*p_priv;
 	const keyspan_device_details	*d_details;
 
-	dbg("keyspan_startup called.\n");
+	/*  dbg("keyspan_startup called."); */
 
 	for (i = 0; (d_details = keyspan_devices[i]) != NULL; ++i)
 		if (d_details->product_id == serial->dev->descriptor.idProduct)
@@ -1704,7 +1624,7 @@ static int keyspan_startup (struct usb_serial *serial)
 	serial->private = kmalloc(sizeof(struct keyspan_serial_private),
 				  GFP_KERNEL);
 	if (!serial->private) {
-		dbg(__FUNCTION__ "kmalloc for keyspan_serial_private failed.\n");
+		dbg(__FUNCTION__ "kmalloc for keyspan_serial_private failed.");
 		return (1);
 	}
 	memset(serial->private, 0, sizeof(struct keyspan_serial_private));
@@ -1718,7 +1638,7 @@ static int keyspan_startup (struct usb_serial *serial)
 		port->private = kmalloc(sizeof(struct keyspan_port_private),
 					GFP_KERNEL);
 		if (!port->private) {
-			dbg(__FUNCTION__ "kmalloc for keyspan_port_private (%d) failed!.\n", i);
+			dbg(__FUNCTION__ "kmalloc for keyspan_port_private (%d) failed!.", i);
 			return (1);
 		}
 		memset(port->private, 0, sizeof(struct keyspan_port_private));
@@ -1730,7 +1650,7 @@ static int keyspan_startup (struct usb_serial *serial)
 
 	s_priv->instat_urb->dev = serial->dev;
 	if ((err = usb_submit_urb(s_priv->instat_urb)) != 0) {
-		dbg(__FUNCTION__ " submit instat urb failed %d\n", err);
+		dbg(__FUNCTION__ " submit instat urb failed %d", err);
 	}
 			
 	return (0);
@@ -1743,7 +1663,7 @@ static void keyspan_shutdown (struct usb_serial *serial)
 	struct keyspan_serial_private 	*s_priv;
 	struct keyspan_port_private	*p_priv;
 
-	dbg("keyspan_shutdown called\n");
+	/*  dbg("keyspan_shutdown called"); */
 
 	s_priv = (struct keyspan_serial_private *)(serial->private);
 
