@@ -1,23 +1,27 @@
 /*
- * linux/include/asm-arm/atomic.h
+ *  linux/include/asm-arm/atomic.h
  *
- * Copyright (c) 1996 Russell King.
+ *  Copyright (c) 1996 Russell King.
  *
- * Changelog:
- *  27-06-1996	RMK	Created
- *  13-04-1997	RMK	Made functions atomic!
- *  07-12-1997	RMK	Upgraded for v2.1.
- *  26-08-1998	PJB	Added #ifdef __KERNEL__
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ *  Changelog:
+ *   27-06-1996	RMK	Created
+ *   13-04-1997	RMK	Made functions atomic!
+ *   07-12-1997	RMK	Upgraded for v2.1.
+ *   26-08-1998	PJB	Added #ifdef __KERNEL__
  */
 #ifndef __ASM_ARM_ATOMIC_H
 #define __ASM_ARM_ATOMIC_H
 
-#ifdef __SMP__
+#include <linux/config.h>
+
+#ifdef CONFIG_SMP
 #error SMP not supported
 #endif
 
-#include <linux/config.h>
-
 #ifdef CONFIG_ARCH_CO285
 typedef struct { volatile int counter; } atomic_t;
 #else
@@ -27,7 +31,7 @@ typedef struct { int counter; } atomic_t;
 #define ATOMIC_INIT(i)	{ (i) }
 
 #ifdef __KERNEL__
-#include <asm/system.h>
+#include <asm/proc/system.h>
 
 #define atomic_read(v)	((v)->counter)
 #define atomic_set(v,i)	(((v)->counter) = (i))
@@ -36,36 +40,36 @@ static __inline__ void atomic_add(int i, volatile atomic_t *v)
 {
 	unsigned long flags;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	v->counter += i;
-	restore_flags (flags);
+	__restore_flags(flags);
 }
 
 static __inline__ void atomic_sub(int i, volatile atomic_t *v)
 {
 	unsigned long flags;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	v->counter -= i;
-	restore_flags (flags);
+	__restore_flags(flags);
 }
 
 static __inline__ void atomic_inc(volatile atomic_t *v)
 {
 	unsigned long flags;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	v->counter += 1;
-	restore_flags (flags);
+	__restore_flags(flags);
 }
 
 static __inline__ void atomic_dec(volatile atomic_t *v)
 {
 	unsigned long flags;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	v->counter -= 1;
-	restore_flags (flags);
+	__restore_flags(flags);
 }
 
 static __inline__ int atomic_dec_and_test(volatile atomic_t *v)
@@ -73,10 +77,10 @@ static __inline__ int atomic_dec_and_test(volatile atomic_t *v)
 	unsigned long flags;
 	int result;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	v->counter -= 1;
 	result = (v->counter == 0);
-	restore_flags (flags);
+	__restore_flags(flags);
 
 	return result;
 }
@@ -86,10 +90,10 @@ extern __inline__ int atomic_add_negative(int i, volatile atomic_t *v)
 	unsigned long flags;
 	int result;
 
-	save_flags_cli(flags);
+	__save_flags_cli(flags);
 	v->counter += i;
 	result = (v->counter < 0);
-	restore_flags(flags);
+	__restore_flags(flags);
 
 	return result;
 }
@@ -98,9 +102,9 @@ static __inline__ void atomic_clear_mask(unsigned long mask, unsigned long *addr
 {
 	unsigned long flags;
 
-	save_flags_cli (flags);
+	__save_flags_cli(flags);
 	*addr &= ~mask;
-	restore_flags (flags);
+	__restore_flags(flags);
 }
 
 #endif
