@@ -42,22 +42,11 @@
 #ifndef _LINUX_INET_H
 #define _LINUX_INET_H
 
-#if defined(__i386__)
-#define NET16(x)	((((x) >> 8) & 0x00FF) | (((x) << 8) & 0xFF00))
-#elif defined(__mc68000__)
-#define NET16(x)	(x)
-#elif defined(__alpha__)
-#define NET16(x)	((((x) >> 8) & 0x00FF) | (((x) << 8) & 0xFF00))
-#else
-#error change this to match your machine
-#endif
-
 #ifdef __KERNEL__
 
-
 extern void		inet_proto_init(struct net_proto *pro);
-extern char		*in_ntoa(unsigned long in);
-extern unsigned long	in_aton(char *str);
+extern char		*in_ntoa(__u32 in);
+extern __u32		in_aton(const char *str);
 
 #endif
 #endif	/* _LINUX_INET_H */
