@@ -58,7 +58,7 @@ It sends the data to the socket indicated by desc->buf.
 static int sock_send_actor(read_descriptor_t * desc, struct page *page, unsigned long offset, unsigned long size)
 {
 	int written;
-	unsigned long kaddr;
+	char *kaddr;
 	unsigned long count = desc->count;
 	struct socket *sock = (struct socket *) desc->buf;
 	mm_segment_t old_fs;
@@ -69,7 +69,7 @@ static int sock_send_actor(read_descriptor_t * desc, struct page *page, unsigned
 	set_fs(KERNEL_DS);
 
 	kaddr = kmap(page);
-	written = SendBuffer_async(sock,(char *)kaddr + offset,size);
+	written = SendBuffer_async(sock, kaddr + offset, size);
 	kunmap(page);
 	set_fs(old_fs);
 	if (written < 0) {
@@ -114,8 +114,7 @@ int DataSending(const int CPUNR)
 			
 			inode = CurrentRequest->filp->f_dentry->d_inode;
 			
-			if ( (inode!=NULL)&&(inode->i_op!=NULL)&&(inode->i_op->readpage!=NULL))
-			{
+			if (inode->i_mapping->a_ops->readpage) {
 				/* This does the actual transfer using sendfile */		
 				read_descriptor_t desc;
 				loff_t *ppos;
@@ -172,7 +171,7 @@ int DataSending(const int CPUNR)
 			if  (CurrentRequest->sock->sk->state == TCP_ESTABLISHED ||
 			     CurrentRequest->sock->sk->state == TCP_CLOSE_WAIT)
 			{
-				CurrentRequest->sock->sk->nonagle = 0;
+				CurrentRequest->sock->sk->tp_pinfo.af_tcp.nonagle = 0;
 				tcp_push_pending_frames(CurrentRequest->sock->sk,&(CurrentRequest->sock->sk->tp_pinfo.af_tcp));
 			}
 			release_sock(CurrentRequest->sock->sk);
